<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
class VAM_UDesign_Admin {

  /**
   * The ID of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param      string    $plugin_name       The name of this plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {

    $this->plugin_name = $plugin_name;
    $this->version = $version;

    $this->load_dependencies();

  }

  /**
   * Load the required dependencies for this plugin.
   *
   * Include the following files that make up the plugin:
   *
   * - VAM_UDesign_Loader. Orchestrates the hooks of the plugin.
   * - VAM_UDesign_i18n. Defines internationalization functionality.
   * - VAM_UDesign_Admin. Defines all hooks for the admin area.
   * - VAM_UDesign_Public. Defines all hooks for the public side of the site.
   *
   * Create an instance of the loader which will be used to register the hooks
   * with WordPress.
   *
   * @since    1.0.0
   * @access   private
   */
  private function load_dependencies() {
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-admin-settings.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-admin-menus.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-admin-post-types.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-clipart-meta-boxes.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-clipart-bulk-upload.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-design-meta-boxes.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-design-import.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-product-import.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-wc-product-attribute-swatches.php';
    require_once plugin_dir_path( __FILE__ ) . 'class-vam-udesign-wc-meta-box-product-data.php';
  }

  /**
   * Register the stylesheets for the admin area.
   *
   * @since    1.0.0
   */
  public function enqueue_styles() {
    wp_register_style( 'jquery-modal', VAM_UDesign()->plugin_url( '/assets/css/jquery.modal.css' ), array(), $this->version, 'all' );
    wp_enqueue_style( $this->plugin_name, VAM_UDesign()->plugin_url( '/assets/css/vam-udesign-admin.css' ), array( 'jquery-modal', 'wp-color-picker' ), $this->version, 'all' );
  }

  /**
   * Register the JavaScript for the admin area.
   *
   * @since    1.0.0
   */
  public function enqueue_scripts() {

    global $pagenow, $post, $post_type;

    if ( empty( $post_type ) && !empty( $_GET['post_type'] ) ) {
      $post_type = $_GET['post_type'];
    }

    // Get admin screen id.
    $screen    = get_current_screen();
    $screen_id = $screen ? $screen->id : '';
    $suffix    = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
    $page      = empty( $_GET['page'] ) ? '' : $_GET['page'];

    //Enqueue media if on a product attribute page
    if ( strpos( $screen->id, 'pa_' ) !== false ) {
      if ( function_exists( 'wp_enqueue_media' ) ) {
        wp_enqueue_media();
      }
    }

    // Product attribute color/image/label swatch
    if( in_array( $pagenow, array( 'term.php', 'edit-tags.php' ) ) && 'product' == $post_type ) {

      wp_enqueue_script( $this->plugin_name . '-admin-product-attribute-swatch', VAM_UDesign()->plugin_url( '/assets/js/product-attribute-swatch' . $suffix . '.js' ), array( 'jquery', 'wp-color-picker', 'media-upload' ), $this->version, false );

      $data = array(
        'placeholder_img_src' => apply_filters( 'woocommerce_placeholder_img_src', WC()->plugin_url() . '/assets/images/placeholder.png' )
      );

      wp_localize_script( $this->plugin_name . '-admin-product-attribute-swatch', 'wc_swatches_params', $data );
    }

    // Meta boxes
    if ( in_array( $screen_id, array( 'product', 'edit-product' ) ) ) {
      wp_enqueue_script( 'jquery-modal', VAM_UDesign()->plugin_url( '/assets/js/jquery-modal/jquery.modal' . $suffix . '.js' ), array(), $this->version, false );
      wp_enqueue_script( 'interactjs', VAM_UDesign()->plugin_url( '/assets/js/interactjs/interact' . $suffix . '.js' ), array(), $this->version, false );
      wp_enqueue_script( $this->plugin_name . '-admin-designer-views-meta-boxes', VAM_UDesign()->plugin_url( '/assets/js/meta-boxes-designer-views' . $suffix . '.js' ), array( 'interactjs', 'wc-admin-variation-meta-boxes', 'wp-color-picker', 'jquery-modal' ), $this->version, false );

      $params = array(
        'post_id'                             => isset( $post->ID ) ? $post->ID : '',
        'ajax_url'                            => admin_url( 'admin-ajax.php' ),
        'woocommerce_placeholder_img_src'     => wc_placeholder_img_src(),
        'load_designer_views_nonce'           => wp_create_nonce( 'load-designer-views' ),
        'save_designer_views_nonce'           => wp_create_nonce( 'save-designer-views' ),
        'save_color_attribute_nonce'          => wp_create_nonce( 'save-color-attribute' ),
        'load_edit_color_attribute_nonce'     => wp_create_nonce( 'load-edit-color-attribute' ),
        'i18n_link_all_variations'            => esc_js( sprintf( __( 'Are you sure you want to link all variations? This will create a new variation for each and every possible combination of variation attributes (max %d per run).', 'vam-udesign' ), defined( 'WC_MAX_LINKED_VARIATIONS' ) ? WC_MAX_LINKED_VARIATIONS : 50 ) ),
        'i18n_enter_a_value'                  => esc_js( __( 'Enter a value', 'vam-udesign' ) ),
        'i18n_last_warning'                   => esc_js( __( 'Last warning, are you sure?', 'vam-udesign' ) ),
        'i18n_choose_image'                   => esc_js( __( 'Choose an image', 'vam-udesign' ) ),
        'i18n_set_image'                      => esc_js( __( 'Set view image', 'vam-udesign' ) ),
        'i18n_edited_designer_views'          => esc_js( __( 'Save changes before changing page?', 'vam-udesign' ) ),
        'i18n_designer_view_count_single'     => esc_js( __( '%qty% view', 'vam-udesign' ) ),
        'i18n_designer_view_count_plural'     => esc_js( __( '%qty% views', 'vam-udesign' ) )
      );

      wp_localize_script( $this->plugin_name . '-admin-designer-views-meta-boxes', 'admin_meta_boxes_designer_views', $params );
    }

    if( in_array( $pagenow, array( 'post.php', 'post-new.php' ) ) && 'clipart' == $post_type ) {
      $this->enqueue_common_scripts();
      wp_enqueue_script( $this->plugin_name . '-admin-clipart-meta-boxes', VAM_UDesign()->plugin_url( '/assets/js/meta-boxes-clipart' . $suffix . '.js' ), array( 'jquery', 'media-upload' ), $this->version, false );

      $clipart_id = empty( $post->ID ) ? '' : $post->ID;
      $params = array(
        'post_id'  => $clipart_id,
        'ajax_url' => admin_url( 'admin-ajax.php' ),
        'security' => array(
          'clipart_nonce' => wp_create_nonce( 'clipart' ),
          'thumb_nonce' => wp_create_nonce( 'post-thumbnail' ),
          'blob_attachment_nonce' => wp_create_nonce( 'blob_attachment' ),
        )
      );

      wp_localize_script( $this->plugin_name . '-admin-clipart-meta-boxes', 'clipart_data', $params );
    }

    if ( in_array( $pagenow, array( 'edit.php' ) )  && 'clipart' == $post_type ) {
      $this->enqueue_common_scripts();
      wp_enqueue_script( $this->plugin_name . '-admin-clipart-bulk-upload', VAM_UDesign()->plugin_url( '/assets/js/clipart-bulk-upload' . $suffix . '.js' ), array( 'jquery', 'post' ), $this->version, false );

      $params = array(
        'ajax_url' => admin_url( 'admin-ajax.php' ),
        'security' => array(
          'clipart_nonce' => wp_create_nonce( 'clipart' ),
          'blob_attachment_nonce' => wp_create_nonce( 'blob_attachment' ),
        )
      );

      wp_localize_script( $this->plugin_name . '-admin-clipart-bulk-upload', 'bulk_upload_data', $params );
    }

    if ( in_array( $pagenow, array( 'post.php', 'post-new.php' ) ) && 'design' == $post_type ) {
      wp_enqueue_script( $this->plugin_name . '-admin-design-meta-boxes', VAM_UDesign()->plugin_url( '/assets/js/meta-boxes-design' . $suffix . '.js' ), array( 'jquery', 'wp-color-picker', 'media-upload' ), $this->version, false );
    }

    // Designs import
    if ( in_array( $pagenow, array( 'edit.php' ) ) && 'design' == $post_type && $page == 'design_import' ) {
      $this->enqueue_common_scripts();
      wp_enqueue_script( $this->plugin_name . '-admin-design-import', VAM_UDesign()->plugin_url( '/assets/js/design-import' . $suffix . '.js' ), array( 'jquery', $this->plugin_name ), $this->version, false );

      $designs_import_data = array(
        'ajax_url' => admin_url( 'admin-ajax.php' ),
        'messages' => array(
          'select' => __( 'Select all', 'vam-udesign' ),
          'deselect' => __( 'Deselect all', 'vam-udesign' ),
          'importing' => __( 'Importing design...', 'vam-udesign' )
        ),
        'security' => array(
          'design_import_nonce' => wp_create_nonce( 'design_import' )
        )
      );

      wp_localize_script( $this->plugin_name . '-admin-design-import', 'designs_import_data', $designs_import_data );
    }

    // Products import
    if ( in_array( $pagenow, array( 'edit.php' ) ) && 'product' == $post_type && $page == 'product_import' ) {
      $this->enqueue_common_scripts();
      wp_enqueue_script( $this->plugin_name . '-admin-product-import', VAM_UDesign()->plugin_url( '/assets/js/product-import' . $suffix . '.js' ), array( 'jquery', $this->plugin_name ), $this->version, false );

      $products_import_data = array(
        'ajax_url' => admin_url( 'admin-ajax.php' ),
        'messages' => array(
          'select' => __( 'Select all', 'vam-udesign' ),
          'deselect' => __( 'Deselect all', 'vam-udesign' ),
          'importing_product' => __( 'Importing product...', 'vam-udesign' ),
          'importing_variation' => __( 'Importing variation', 'vam-udesign' )
        ),
        'security' => array(
          'product_import_nonce' => wp_create_nonce( 'product_import' )
        )
      );

      wp_localize_script( $this->plugin_name . '-admin-product-import', 'products_import_data', $products_import_data );
    }
  }

  public function enqueue_common_scripts() {
    $suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

    // enqueue uDesign helper script
    $dependencies = array( 'jquery', 'jquery-blockui', 'canvas-to-blob' );
    if ( is_admin() ) {
      $dependencies[] = 'post';
    }

    wp_enqueue_script( 'canvas-to-blob', VAM_UDesign()->plugin_url( '/assets/js/canvas-to-blob/canvas-to-blob' . $suffix . '.js' ), array(), $this->version, false );
    wp_enqueue_script( $this->plugin_name, VAM_UDesign()->plugin_url( '/assets/js/vam-udesign' . $suffix . '.js' ), $dependencies, $this->version, false );

    $params = array(
      'ajax_url' => admin_url( 'admin-ajax.php' ),
    );

    wp_localize_script( $this->plugin_name, 'vam_udesign_params', $params );
  }

  public function enqueue_designer_styles() {
    wp_enqueue_style( 'dropzone', VAM_UDesign()->plugin_url( '/assets/css/dropzone/dropzone.css' ), array(), $this->version, 'all' );
    wp_enqueue_style( 'designer', VAM_UDesign()->plugin_url( '/assets/css/designer.css' ), array(), $this->version, 'all' );
  }

  public function enqueue_designer_scripts() {
    global $pagenow, $post, $post_type;
    $suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

    $this->enqueue_common_scripts();
    wp_enqueue_script( 'designer-api', ( UDESIGN_DEBUG ? UDESIGN_DEBUG_URL : UDESIGN_PRODUCTION_URL ) . '/api.js', array(), UDESIGN_VERSION, false );
    wp_enqueue_script( 'dropzone', VAM_UDesign()->plugin_url( '/assets/js/dropzone/dropzone' . $suffix . '.js' ), array(), $this->version, false );
    wp_enqueue_script( 'designer-init', VAM_UDesign()->plugin_url( '/assets/js/designer-init' . $suffix . '.js' ), array( 'jquery', $this->plugin_name, 'designer-api', 'dropzone' ), $this->version, false );

    // enqueue order designer script and data
    if ( $post_type == 'shop_order' ) {
      wp_enqueue_script( 'file-saver', VAM_UDesign()->plugin_url( '/assets/js/file-saver/FileSaver' . $suffix . '.js' ), array(), $this->version, false );
      wp_enqueue_script( 'designer-shop-order', VAM_UDesign()->plugin_url( '/assets/js/designer-shop-order' . $suffix . '.js' ), array( 'designer-init', 'file-saver' ), $this->version, false );

      $order = wc_get_order( $post->ID );
      if ( ! empty( $order ) ) {
        $items = array();

        $order_items = $order->get_items();
        foreach ( $order_items as $order_item_id => &$order_item ) {
          if ( empty( wc_get_order_item_meta( $order_item_id, 'is_designer_product', true ) ) ) {
            continue;
          }

          $design_meta = wc_get_order_item_meta( $order_item_id, 'design_meta', true );
          if ( ! empty ( $design_meta ) ) {
            $items[$order_item_id] = array(
              'product_id' => VAM_UDesign_WC_Compat::get_prop( $order_item, 'variation_id' ),
              'design' => vam_udesign_json_decode( $design_meta )
            );
          }
        }

        $designer_order_data = array(
          'post_id' => $post->ID,
          'order_items' => $items,
          'security' => array(
            'designer_download_nonce' => wp_create_nonce( 'designer_download' )
          ),
        );

        wp_localize_script( 'designer-shop-order', 'designer_order_data', $designer_order_data );
      }
    }

    // retrieve product to load
    $product_id = 0;
    if ( ! empty( $_GET['product_id'] ) ) {
      $product_id = absint( $_GET['product_id'] );

      $product = wc_get_product( $product_id );
      if ( empty( $product ) || 'variation' != $product->get_type() ) {
        $product_id = 0;
      }
    }

    if ( empty( $product_id ) ) {
      $product_id = absint( get_post_meta( $post->ID, '_design_default_product_id', true ) );

      $product = wc_get_product( $product_id );
      if ( empty( $product ) || 'variation' != $product->get_type() ) {
        $product_id = 0;
      }
    }

    if ( empty( $product_id ) ) {
      $default_product_id = absint( get_option( 'udesign_default_product_id', '' ) );
      if ( ! empty( $default_product_id ) ) {
        $product_id = vam_udesign_get_product_default_variation( $default_product_id );
      }
    }

    $size   = '';
    $color  = '';
    if ( ! empty( $product_id ) ) {
      $size         = get_post_meta( $product_id, 'attribute_pa_size', true );
      $color_slug   = get_post_meta( $product_id, 'attribute_pa_color', true );
      $color_term   = get_term_by( 'slug', $color_slug, 'pa_color' );
      $swatch_term  = new VAM_UDesign_WC_Swatch_Term( $color_term->term_id );
      $color        = $swatch_term->get_color();
    }

    // retrieve design to load
    $design_id = false;
    if ( 'design' == $post_type && is_admin() ) {
      $design_id = $post->ID;
    } else if ( ! empty( $_GET['design_id'] ) ) {
      $design_id = absint( $_GET['design_id'] );
    }

    $design = array( 'layers' => array() );
    if ( ! empty( $design_id ) ) {
      $design_meta = get_post_meta( $design_id, '_design_meta', true );
      if ( ! empty ( $design_meta ) ) {
        $design = vam_udesign_json_decode( $design_meta );
      }
    } else if ( ! empty( $_GET['order_id'] ) && ! empty( $_GET['order_item_id'] ) ) {
      $order_id = absint( $_GET['order_id'] );

      $order = wc_get_order( $order_id );
      if ( ! empty( $order ) ) {
        $user_id = absint( VAM_UDesign_WC_Compat::get_prop( $order, 'user_id' ) );

        if ( $user_id == get_current_user_id() ) {
          $order_item_id = absint( $_GET['order_item_id'] );

          $order_items = $order->get_items();
          foreach ( $order_items as $item_id => &$order_item ) {
            if ( $item_id != $order_item_id ) {
              continue;
            }

            $design_meta = wc_get_order_item_meta( $order_item_id, 'design_meta', true );
            if ( ! empty ( $design_meta ) ) {
              $design = vam_udesign_json_decode( $design_meta );
            }
          }
        }
      }
    }

    // localize designer data
    $designer_config = json_encode( array(
      'debug'   => UDESIGN_DEBUG,
      'url'     => UDESIGN_DEBUG ? UDESIGN_DEBUG_URL : UDESIGN_PRODUCTION_URL,
      'version' => UDESIGN_VERSION,
      'license' => get_option( 'udesign_api_key', '' )
    ) );

    $config = array(
      'product_categories' => json_encode( VAM_UDesign_Ajax::designer_product_categories() ),
      'clipart_categories' => json_encode( VAM_UDesign_Ajax::clipart_categories() ),
      'design_categories'  => json_encode( VAM_UDesign_Ajax::design_categories() ),
    );

    $is_admin = is_admin();
    $allow_clipart_upload = $is_admin ? true :
      get_option( 'udesign_customer_clipart_upload', 'enabled' ) == 'enabled';

    $params = array(
      'post_id'              => isset( $post->ID ) ? $post->ID : '',
      'product_id'           => $product_id,
      'size'                 => $size,
      'color'                => $color,
      'design_id'            => $design_id,
      'design'               => $design,
      'ajax_url'             => admin_url( 'admin-ajax.php' ),
      'security'             => array(
        'designer_nonce' => wp_create_nonce( 'designer' )
      ),
      'i18n_edited_design'   => esc_js( __( 'Are you sure you want to leave the page?', 'vam-udesign' ) ),
      'designer_config'      => $designer_config,
      'config'               => $config,
      'is_admin'             => $is_admin,
      'allow_clipart_upload' => $allow_clipart_upload
    );
    wp_localize_script( 'designer-init', 'designer_data', $params );
  }

}
