/* jshint expr: true */
/* global design_gallery_params */
;(function ( $, window, document, undefined ) {

  // design_gallery_params is required to continue, ensure the object exists
  if ( typeof design_gallery_params === 'undefined' ) {
    return false;
  }

  /**
   * Product gallery class.
   */
  var DesignGallery = function( $target, args ) {
    this.$target = $target;
    this.$images = $( '.woocommerce-product-gallery__image', $target );

    // No images? Abort.
    if ( 0 === this.$images.length ) {
      this.$target.css( 'opacity', 1 );
      return;
    }

    // Save default gallery for reset.
    this.$design = $target.closest( '.design' );
    if ( !this.$design.data( 'default_preview_html' ) ) {
      this.$design.data( 'default_preview_html', $target.outerHTML() );
    }

    // Make this object available.
    $target.data( 'design_gallery', this );

    // Pick functionality to initialize...
    this.flexslider_enabled = $.isFunction( $.fn.flexslider ) && design_gallery_params.flexslider_enabled;
    this.zoom_enabled       = $.isFunction( $.fn.zoom ) && design_gallery_params.zoom_enabled;
    this.photoswipe_enabled = typeof PhotoSwipe !== 'undefined' && design_gallery_params.photoswipe_enabled;

    // ...also taking args into account.
    if ( args ) {
      this.flexslider_enabled = false === args.flexslider_enabled ? false : this.flexslider_enabled;
      this.zoom_enabled       = false === args.zoom_enabled ? false : this.zoom_enabled;
      this.photoswipe_enabled = false === args.photoswipe_enabled ? false : this.photoswipe_enabled;
    }

    // Bind functions to this.
    this.initFlexslider       = this.initFlexslider.bind( this );
    this.initZoom             = this.initZoom.bind( this );
    this.initPhotoswipe       = this.initPhotoswipe.bind( this );
    this.onResetSlidePosition = this.onResetSlidePosition.bind( this );
    this.getGalleryItems      = this.getGalleryItems.bind( this );
    this.openPhotoswipe       = this.openPhotoswipe.bind( this );

    if ( this.flexslider_enabled ) {
      this.initFlexslider();
      $target.on( 'woocommerce_gallery_reset_slide_position', this.onResetSlidePosition );
    } else {
      this.$target.css( 'opacity', 1 );
    }

    if ( this.zoom_enabled ) {
      this.initZoom();
      $target.on( 'woocommerce_gallery_init_zoom', this.initZoom );
    }

    if ( this.photoswipe_enabled ) {
      this.initPhotoswipe();
    }
  };

  /**
   * Initialize flexSlider.
   */
  DesignGallery.prototype.initFlexslider = function() {
    var images  = this.$images,
      $target = this.$target;
    $target.flexslider( {
      selector:       '.woocommerce-product-gallery__wrapper > .woocommerce-product-gallery__image',
      animation:      design_gallery_params.flexslider.animation,
      smoothHeight:   design_gallery_params.flexslider.smoothHeight,
      directionNav:   design_gallery_params.flexslider.directionNav,
      controlNav:     design_gallery_params.flexslider.controlNav,
      slideshow:      design_gallery_params.flexslider.slideshow,
      animationSpeed: design_gallery_params.flexslider.animationSpeed,
      animationLoop:  design_gallery_params.flexslider.animationLoop, // Breaks photoswipe pagination if true.
      start: function() {
        $target.css( 'opacity', 1 );

        var largest_height = 0;

        images.each( function() {
          var height = $( this ).height();

          if ( height > largest_height ) {
            largest_height = height;
          }
        } );

        images.each( function( index, element ) {
          var container = $( this );
          container.css( 'min-height', largest_height );

          var background = container.data('background');
          if ( background ) {
            container.find( 'img' ).css( 'background-color', background );
            container.closest( '.woocommerce-design-gallery' )
              .find( '.flex-control-nav li img' ).eq( index )
              .css( 'background-color', background );
          }
        } );
      }
    } );
  };

  /**
   * Init zoom.
   */
  DesignGallery.prototype.initZoom = function() {
    var zoomTarget   = this.$images,
      galleryWidth = this.$target.width(),
      zoomEnabled  = false;

    if ( ! this.flexslider_enabled ) {
      zoomTarget = zoomTarget.first();
    }

    $( zoomTarget ).each( function( index, target ) {
      var image = $( target ).find( 'img' );

      if ( image.data( 'large_image_width' ) > galleryWidth ) {
        zoomEnabled = true;
        return false;
      }
    } );

    // But only zoom if the img is larger than its container.
    if ( zoomEnabled ) {
      var zoom_options = {
        touch: false
      };

      if ( 'ontouchstart' in window ) {
        zoom_options.on = 'click';
      }

      zoomTarget.trigger( 'zoom.destroy' );
      zoomTarget.zoom( zoom_options );
    }
  };

  /**
   * Init PhotoSwipe.
   */
  DesignGallery.prototype.initPhotoswipe = function() {
    if ( this.zoom_enabled && this.$images.length > 0 ) {
      this.$target.prepend( '<a href="#" class="woocommerce-product-gallery__trigger">🔍</a>' );
      this.$target.on( 'click', '.woocommerce-product-gallery__trigger', this.openPhotoswipe );
    }
    this.$target.on( 'click', '.woocommerce-product-gallery__image a', this.openPhotoswipe );
  };

  /**
   * Reset slide position to 0.
   */
  DesignGallery.prototype.onResetSlidePosition = function() {
    this.$target.flexslider( 0 );
  };

  /**
   * Get product gallery image items.
   */
  DesignGallery.prototype.getGalleryItems = function() {
    var $slides = this.$images,
      items   = [];

    if ( $slides.length > 0 ) {
      $slides.each( function( i, el ) {
        var img = $( el ).find( 'img' ),
          large_image_src = img.attr( 'data-large_image' ),
          large_image_w   = img.attr( 'data-large_image_width' ),
          large_image_h   = img.attr( 'data-large_image_height' ),
          item            = {
            src: large_image_src,
            w:   large_image_w,
            h:   large_image_h,
            title: img.attr( 'title' )
          };
        items.push( item );
      } );
    }

    return items;
  };

  /**
   * Open photoswipe modal.
   */
  DesignGallery.prototype.openPhotoswipe = function( e ) {
    e.preventDefault();

    var pswpElement = $( '.pswp' )[0],
      items       = this.getGalleryItems(),
      eventTarget = $( e.target ),
      clicked;

    if ( ! eventTarget.is( '.woocommerce-product-gallery__trigger' ) ) {
      clicked = eventTarget.closest( '.woocommerce-product-gallery__image' );
    } else {
      clicked = this.$target.find( '.flex-active-slide' );
    }

    var options = {
      index:                 $( clicked ).index(),
      shareEl:               false,
      closeOnScroll:         false,
      history:               false,
      hideAnimationDuration: 0,
      showAnimationDuration: 0
    };

    // Initializes and opens PhotoSwipe.
    var photoswipe = new PhotoSwipe( pswpElement, PhotoSwipeUI_Default, items, options );
    photoswipe.init();
  };

  /**
   * Function to call wc_design_gallery on jquery selector.
   */
  $.fn.wc_design_gallery = function( args ) {
    new DesignGallery( this, args );
    return this;
  };

  $.fn.outerHTML = function() {
    return (this[0]) ? this[0].outerHTML : '';
  };

  /*
   * Initialize all galleries on page.
   */
  $( '.woocommerce-design-gallery' ).each( function() {
    $( this ).wc_design_gallery();
  } );
})( jQuery, window, document );
