/* global single_design_params */
jQuery( function( $ ) {
  'use strict';

  // single_design_params is required to continue, ensure the object exists
  if ( typeof single_design_params === 'undefined' ) {
    return false;
  }

  var design_selector = '.design[itemtype="Product"]';
  var product_list_selector = '.designer_products';
  var product_item_selector = '.designer_product';
  var product_cat_class_selector = '.product_cat_id_';
  var product_cat_dropdown_selector = '.dropdown_product_cat';
  var variations_form_selector = '.design_variations_form';

  /**
   * Single design actions
   */
  var single_design_actions = {

    /**
     * Initialize actions
     */
    init: function() {
      $( design_selector )
        .on( 'click', product_item_selector, this.load_designer_product )
        .on( 'change', product_cat_dropdown_selector, this.change_product_cat )
        .on( 'designer_product_loaded', this.designer_product_loaded );
      $( '.designer-wrapper' )
        .on( 'load_designer_product', this.load_designer_product );
      $( window ).on( 'load', this.init_products_layout );
    },

    /**
     * Load designer product
     *
     * @param {Object} event
     */
    load_designer_product: function( event ) {
      event.preventDefault();

      var product = $( this );
      var productID = product.data( 'product_id' );
      var variations_form = $( variations_form_selector );
      if ( !productID ) {
        return false;
      }

      if ( product.is('.designer-wrapper') ) {
        return single_design_ajax.load_designer_product( productID );
      }

      if ( product.is( '.selected ') && variations_form.length &&
        variations_form.data( 'product_id' ) == productID ) {
        return false;
      }

      $( product_list_selector ).find( product_item_selector ).removeClass( 'selected' );
      product.addClass( 'selected' );

      single_design_ajax.load_designer_product( productID );
    },

    /**
     * Initiate designer products layout
     */
    init_products_layout: function() {
      var wrapper = $( design_selector );
      var container = wrapper.find( product_list_selector );
      if ( !container.length ) {
        return;
      }

      container.on( 'layoutComplete', function( event, filteredItems ) {
        if ( !filteredItems.length ) {
          return;
        }

        $( filteredItems.shift().element ).trigger( 'click' );
      });

      var cat_id = parseInt( wrapper.find( product_cat_dropdown_selector ).val(), 10 );
      if ( 'masonry' === single_design_params.products_list_layout && $().isotope ) {
        container.isotope({
          itemSelector: product_item_selector,
          filter: cat_id ? product_cat_class_selector + cat_id : '*',
          resizable: true,
          containerStyle : {
            overflow : 'hidden',
            position : 'relative'
          },
          layoutMode: 'masonry',
          masonry: {
            columnWidth: product_item_selector
          }
        });
      } else if ( 'carousel' === single_design_params.products_list_layout && $().owlCarousel ) {
        if ( !cat_id ) {
          return;
        }

        container.addClass( 'owl-carousel owl-theme' ).owlCarousel({
          nav: true,
          dots: false,
          navText: [
            '<i class="vam-udesign-owl-nav-control icon-angle-left"></i>',
            '<i class="vam-udesign-owl-nav-control icon-angle-right"></i>'
          ]
        });
      } else {
        if ( !cat_id ) {
          return;
        }

        container.find( product_item_selector ).hide()
          .filter( product_cat_class_selector + cat_id ).show();
      }

      wrapper.find( product_item_selector ).each( function( i ) {
        $(this).delay( i * 150 ).animate( {'opacity' : 1}, 500 );
      });
    },

    /**
     * Change designer product category
     *
     * @param {Object} event
     */
    change_product_cat: function( event ) {
      var wrapper = $( design_selector );
      var container = wrapper.find( product_list_selector );
      if ( !container.length ) {
        return;
      }

      var cat_id = parseInt( $( product_cat_dropdown_selector ).val(), 10 );
      if ( 'masonry' === single_design_params.products_list_layout && $().isotope ) {
        wrapper.find( product_list_selector ).isotope( {
          filter: cat_id ? product_cat_class_selector + cat_id : '*'
        } );
      } else if ( 'carousel' === single_design_params.products_list_layout && $().owlCarousel ) {
        if ( !cat_id ) {
          return;
        }

        single_design_ajax.load_product_gallery( cat_id );
      } else {
        if ( !cat_id ) {
          return;
        }

        container.find( product_item_selector ).hide()
          .filter( product_cat_class_selector + cat_id ).show();
        container.trigger( 'layoutComplete',
          [[{element: container.find( product_item_selector).filter( ':visible' ).first()}]] );
      }
    },

    /**
     * Run actions when designer product is loaded
     *
     * @param {Object} event
     */
    designer_product_loaded: function( event ) {
      $( document.body ).trigger( 'init_single_customize_button' );
      if ( typeof design_add_to_cart_params !== 'undefined' ) {
        $( '.variations_form' ).each( function() {
          $( this ).wc_variation_form();
        });
      }
    }
  };

  /**
   * Single design ajax methods
   */
  var single_design_ajax = {

    /**
     * Initialize single design ajax methods
     */
    init: function() {

    },

    /**
     * Block edit screen
     */
    block: function( selector ) {
      selector = selector || design_selector;
      $( selector ).block({
        message: null,
        overlayCSS: {
          background: '#fff',
          opacity: 0.6
        }
      });
    },

    /**
     * Unblock edit screen
     */
    unblock: function( selector ) {
      selector = selector || design_selector;
      $( selector ).unblock();
    },

    /**
     * Load designer product via Ajax
     *
     */
    load_designer_product: function( product_id ) {
      var wrapper = $( design_selector ).find( '.variations_form' );

      single_design_ajax.block();
      $.ajax({
        url: single_design_params.ajax_url,
        data: {
          action:     'udesign_load_designer_product',
          security:   single_design_params.load_designer_product_nonce,
          design_id:  single_design_params.design_id,
          product_id: product_id
        },
        type: 'POST',
        success: function( response ) {
          wrapper.replaceWith( response );
          $( design_selector ).trigger( 'designer_product_loaded' );
          single_design_ajax.unblock();
        }
      });
    },

    load_product_gallery: function( category_id ) {
      if ( !category_id ) {
        return;
      }

      var wrapper = $( design_selector ).find( product_list_selector );

      single_design_ajax.block();
      $.ajax({
        url: single_design_params.ajax_url,
        data: {
          action:     'udesign_get_product_gallery',
          security:   single_design_params.load_designer_product_nonce,
          category_id:  category_id
        },
        type: 'POST',
        success: function( response ) {
          wrapper.replaceWith( response );
          $( product_list_selector ).find( product_item_selector)
            .first().trigger( 'click' );
          single_design_actions.init_products_layout();
          single_design_ajax.unblock();
        }
      });
    }
  };

  single_design_actions.init();
  single_design_ajax.init();

});
