<?php
/**
 * Design Class
 *
 * Handles individual design data.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

class VAM_UDesign_Design {

  /**
   * The design (post) ID.
   *
   * @var int
   */
  public $id = 0;

  /**
   * $post Stores post data.
   *
   * @var $post WP_Post
   */
  public $post = null;

  /**
   * Constructor gets the post object and sets the ID for the loaded design.
   *
   * @param int|VAM_UDesign_Design|object $design Design ID, post object, or design object
   */
  public function __construct( $design ) {
    if ( is_numeric( $design ) ) {
      $this->id   = absint( $design );
      $this->post = get_post( $this->id );
    } elseif ( $design instanceof VAM_UDesign_Design ) {
      $this->id   = absint( $design->id );
      $this->post = $design->post;
    } elseif ( isset( $design->ID ) ) {
      $this->id   = absint( $design->ID );
      $this->post = $design;
    }
  }

  /**
   * __isset function.
   *
   * @param mixed $key
   * @return bool
   */
  public function __isset( $key ) {
    return metadata_exists( 'post', $this->id, '_design_' . $key );
  }

  /**
   * __get function.
   *
   * @param string $key
   * @return mixed
   */
  public function __get( $key ) {
    $value = get_post_meta( $this->id, '_design_' . $key, true );

    if ( in_array( $key, array( 'archive_image', 'single_image', 'thumbnail_image', 'full_image' ) ) ) {
      $value = $value ? esc_url( $value ) : '';
    }

    if ( false !== $value ) {
      $this->$key = $value;
    }

    return $value;
  }

  /**
   * Get the design's post data.
   *
   * @return object
   */
  public function get_post_data() {
    return $this->post;
  }

  /**
   * Return the design ID
   *
   * @since 1.0.0
   * @return int design (post) ID
   */
  public function get_id() {
    return $this->id;
  }

  /**
   * Returns the gallery attachment ids.
   *
   * @return array
   */
  public function get_gallery_attachment_ids() {
    return apply_filters( 'vam_udesign_design_gallery_attachment_ids', array_filter( array_filter( (array) explode( ',', $this->design_image_gallery ) ), 'wp_attachment_is_image' ), $this );
  }

  /**
   * Wrapper for get_permalink.
   *
   * @return string
   */
  public function get_permalink() {
    return get_permalink( $this->id );
  }

  /**
   * Returns whether or not the design post exists.
   *
   * @return bool
   */
  public function exists() {
    return empty( $this->post ) ? false : true;
  }

  /**
   * Get the title of the post.
   *
   * @return string
   */
  public function get_title() {
    return apply_filters( 'vam_udesign_design_title', $this->post ? $this->post->post_title : '', $this );
  }

  /**
   * Get the add to cart button text for the single page.
   *
   * @return string
   */
  public function single_add_to_cart_text() {
    return apply_filters( 'vam_udesign_design_single_add_to_cart_text', __( 'Add to cart', 'vam-udesign' ), $this );
  }

  /**
   * Get the add to cart button text.
   *
   * @return string
   */
  public function add_to_cart_text() {
    return apply_filters( 'vam_udesign_design_add_to_cart_text', __( 'Select options', 'vam-udesign' ), $this );
  }

  /**
   * Returns whether or not the design is featured.
   *
   * @return bool
   */
  public function is_featured() {
    return $this->featured === 'yes' ? true : false;
  }

  /**
   * Returns whether or not the design is visible in the catalog.
   *
   * @return bool
   */
  public function is_visible() {
    if ( ! $this->post ) {
      $visible = false;
    // Published/private
    } elseif ( $this->post->post_status !== 'publish' && ! current_user_can( 'edit_post', $this->id ) ) {
      $visible = false;
    } elseif ( 'hidden' === $this->visibility ) {
      $visible = false;
    } elseif ( 'visible' === $this->visibility ) {
      $visible = true;
    // Visibility in loop
    } elseif ( is_search() ) {
      $visible = 'search' === $this->visibility;
    } else {
      $visible = 'catalog' === $this->visibility;
    }

    return apply_filters( 'vam_udesign_design_is_visible', $visible, $this->id );
  }

  /**
   * Returns the design categories.
   *
   * @param string $sep (default: ', ')
   * @param string $before (default: '')
   * @param string $after (default: '')
   * @return string
   */
  public function get_categories( $sep = ', ', $before = '', $after = '' ) {
    return get_the_term_list( $this->id, 'design_cat', $before, $sep, $after );
  }

  /**
   * Returns the design tags.
   *
   * @param string $sep (default: ', ')
   * @param string $before (default: '')
   * @param string $after (default: '')
   * @return array
   */
  public function get_tags( $sep = ', ', $before = '', $after = '' ) {
    return get_the_term_list( $this->id, 'design_tag', $before, $sep, $after );
  }

  /**
   * Get and return related designs.
   *
   * Notes:
   *  - Results are cached in a transient for faster queries.
   *  - To make results appear random, we query and extra 10 designs and shuffle them.
   *  - To ensure we always have enough results, it will check $limit before returning the cached result, if not recalc.
   *  - This used to rely on transient version to invalidate cache, but to avoid multiple transients we now just expire daily.
   *    This means if a related design is edited and no longer related, it won't be removed for 24 hours. Acceptable trade-off for performance.
   *  - Saving a design will flush caches for that design.
   *
   * @param int $limit (default: 5) Should be an integer greater than 0.
   * @return array Array of post IDs
   */
  public function get_related( $limit = 5 ) {
    global $wpdb;

    $transient_name = 'vam_udesign_related_' . $this->id;
    $related_posts  = get_transient( $transient_name );
    $limit          = $limit > 0 ? $limit : 5;

    // We want to query related posts if they are not cached, or we don't have enough
    if ( false === $related_posts || sizeof( $related_posts ) < $limit ) {
      // Related designs are found from category and tag
      $tags_array = $this->get_related_terms( 'design_tag' );
      $cats_array = $this->get_related_terms( 'design_cat' );

      // Don't bother if none are set
      if ( 1 === sizeof( $cats_array ) && 1 === sizeof( $tags_array )) {
        $related_posts = array();
      } else {
        // Sanitize
        $exclude_ids = array_map( 'absint', array( 0, $this->id ) );

        // Generate query - but query an extra 10 results to give the appearance of random results
        $query = $this->build_related_query( $cats_array, $tags_array, $exclude_ids, $limit + 10 );

        // Get the posts
        $related_posts = $wpdb->get_col( implode( ' ', $query ) );
      }

      set_transient( $transient_name, $related_posts, DAY_IN_SECONDS );
    }

    // Randomise the results
    shuffle( $related_posts );

    // Limit the returned results
    return array_slice( $related_posts, 0, $limit );
  }

  /**
   * Gets the main design image ID.
   *
   * @return int
   */
  public function get_image_id() {
    if ( has_post_thumbnail( $this->id ) ) {
      $image_id = get_post_thumbnail_id( $this->id );
    } elseif ( ( $parent_id = wp_get_post_parent_id( $this->id ) ) && has_post_thumbnail( $parent_id ) ) {
      $image_id = get_post_thumbnail_id( $parent_id );
    } else {
      $image_id = 0;
    }

    return $image_id;
  }

  /**
   * Returns the main design image.
   *
   * @param string $size (default: 'udesign_design_thumbnail')
   * @param array $attr
   * @param bool True to return $placeholder if no image is found, or false to return an empty string.
   * @return string
   */
  public function get_image( $size = 'udesign_design_thumbnail', $attr = array(), $placeholder = true ) {
    $size = vam_udesign_get_image_size_array( $size );

    if ( has_post_thumbnail( $this->id ) ) {
      $image = get_the_post_thumbnail( $this->id, $size, $attr );
    } elseif ( ( $parent_id = wp_get_post_parent_id( $this->id ) ) && has_post_thumbnail( $parent_id ) ) {
      $image = get_the_post_thumbnail( $parent_id, $size, $attr );
    } elseif ( $placeholder ) {
      $image = wc_placeholder_img( $size );
    } else {
      $image = '';
    }

    return $image;
  }

  /**
   * Retrieves related design terms.
   *
   * @param string $term
   * @return array
   */
  protected function get_related_terms( $term ) {
    $terms_array = array(0);

    $terms = apply_filters( 'vam_udesign_get_related_' . $term . '_terms', wp_get_post_terms( $this->id, $term ), $this->id );
    foreach ( $terms as $term ) {
      $terms_array[] = $term->term_id;
    }

    return array_map( 'absint', $terms_array );
  }

  /**
   * Builds the related posts query.
   *
   * @param array $cats_array
   * @param array $tags_array
   * @param array $exclude_ids
   * @param int   $limit
   * @return string
   */
  protected function build_related_query( $cats_array, $tags_array, $exclude_ids, $limit ) {
    global $wpdb;

    $limit = absint( $limit );

    $query           = array();
    $query['fields'] = "SELECT DISTINCT ID FROM {$wpdb->posts} p";
    $query['join']   = " INNER JOIN {$wpdb->postmeta} pm ON ( pm.post_id = p.ID )";
    $query['join']  .= " INNER JOIN {$wpdb->term_relationships} tr ON (p.ID = tr.object_id)";
    $query['join']  .= " INNER JOIN {$wpdb->term_taxonomy} tt ON (tr.term_taxonomy_id = tt.term_taxonomy_id)";
    $query['join']  .= " INNER JOIN {$wpdb->terms} t ON (t.term_id = tt.term_id)";
    $query['where']  = " WHERE 1=1";
    $query['where'] .= " AND p.post_status = 'publish'";
    $query['where'] .= " AND p.post_type = 'design'";
    $query['where'] .= " AND p.ID NOT IN ( " . implode( ',', $exclude_ids ) . " )";
    $relate_by_category = apply_filters( 'vam_udesign_design_related_posts_relate_by_category', true, $this->id );
    $relate_by_tag      = apply_filters( 'vam_udesign_design_related_posts_relate_by_tag', true, $this->id );

    if ( $relate_by_category || $relate_by_tag ) {
      $query['where'] .= ' AND (';

      if ( $relate_by_category ) {
        $query['where'] .= " ( tt.taxonomy = 'design_cat' AND t.term_id IN ( " . implode( ',', $cats_array ) . " ) ) ";
        if ( $relate_by_tag ) {
          $query['where'] .= ' OR ';
        }
      }

      if ( $relate_by_tag ) {
        $query['where'] .= " ( tt.taxonomy = 'design_tag' AND t.term_id IN ( " . implode( ',', $tags_array ) . " ) ) ";
      }

      $query['where'] .= ')';
    }

    $query['limits'] = " LIMIT {$limit} ";
    $query           = apply_filters( 'vam_udesign_design_related_posts_query', $query, $this->id );

    return $query;
  }

  /**
   * Returns the design meta.
   *
   * @return string meta
   */
  public function get_meta() {
    return apply_filters( 'vam_udesign_get_meta', $this->meta, $this );
  }

  /**
   * Returns the design default product id.
   *
   * @return string default product id
   */
  public function get_default_product_id() {
    return apply_filters( 'vam_udesign_get_default_product_id', $this->default_product_id, $this );
  }

  /**
   * Returns the design background color.
   *
   * @return string background color
   */
  public function get_background_color() {
    return apply_filters( 'vam_udesign_get_background_color', $this->background_color, $this );
  }

  /**
   * Returns the design disable customize option.
   *
   * @return int disable customize option
   */
  public function get_disable_customize() {
    return apply_filters( 'vam_udesign_get_disable_customize', absint( $this->disable_customize ), $this );
  }

  /**
   * Returns the design archive image.
   *
   * @return string archive image
   */
  public function get_archive_image() {
    return apply_filters( 'vam_udesign_get_archive_image', $this->archive_image, $this );
  }

  /**
   * Returns the design single image.
   *
   * @return string single image
   */
  public function get_single_image() {
    return apply_filters( 'vam_udesign_get_single_image', $this->single_image, $this );
  }

  /**
   * Returns the design thumbnail image.
   *
   * @return string thumbnail image
   */
  public function get_thumbnail_image() {
    return apply_filters( 'vam_udesign_get_thumbnail_image', $this->thumbnail_image, $this );
  }

  /**
   * Returns the design full image.
   *
   * @return string full image
   */
  public function get_full_image() {
    return apply_filters( 'vam_udesign_get_full_image', $this->full_image, $this );
  }

  /**
   * Returns the design featured option.
   *
   * @return string featured
   */
  public function get_featured() {
    return apply_filters( 'vam_udesign_get_featured', $this->featured, $this );
  }
}
