<?php
/**
 * Contains the query functions which alter the front-end post queries and loops
 *
 * @class     VAM_UDesign_Query
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * VAM_UDesign_Query Class.
 */
class VAM_UDesign_Query {

  /** @public array Query vars to add to wp */
  public $query_vars = array();

  /**
   * Stores chosen attributes
   * @var array
   */
  private static $_chosen_attributes;

  /**
   * Constructor for the query class. Hooks in methods.
   *
   * @access public
   */
  public function __construct() {
    if ( ! is_admin() ) {
      add_filter( 'query_vars', array( $this, 'add_query_vars'), 0 );
      add_action( 'parse_request', array( $this, 'parse_request'), 0 );
      add_action( 'pre_get_posts', array( $this, 'pre_get_posts' ) );
    }
  }

  /**
   * Add query vars.
   *
   * @access public
   * @param array $vars
   * @return array
   */
  public function add_query_vars( $vars ) {
    foreach ( $this->query_vars as $key => $var ) {
      $vars[] = $key;
    }
    return $vars;
  }

  /**
   * Get query vars.
   *
   * @return array
   */
  public function get_query_vars() {
    return $this->query_vars;
  }

  /**
   * Parse the request and look for query vars - endpoints may not be supported.
   */
  public function parse_request() {
    global $wp;

    // Map query vars to their keys, or get them if endpoints are not supported
    foreach ( $this->query_vars as $key => $var ) {
      if ( isset( $_GET[ $var ] ) ) {
        $wp->query_vars[ $key ] = $_GET[ $var ];
      }

      elseif ( isset( $wp->query_vars[ $var ] ) ) {
        $wp->query_vars[ $key ] = $wp->query_vars[ $var ];
      }
    }
  }

  /**
   * Hook into pre_get_posts to do the main design query.
   *
   * @param mixed $q query object
   */
  public function pre_get_posts( $q ) {
    // We only want to affect the main query
    if ( ! $q->is_main_query() ) {
      return;
    }

    // Fix for verbose page rules
    if ( $GLOBALS['wp_rewrite']->use_verbose_page_rules && isset( $q->queried_object->ID ) && $q->queried_object->ID === absint( get_option( 'udesign_designs_page_id', 0 ) ) ) {
      $q->set( 'post_type', 'design' );
      $q->set( 'page', '' );
      $q->set( 'pagename', '' );

      // Fix conditional Functions
      $q->is_archive           = true;
      $q->is_post_type_archive = true;
      $q->is_singular          = false;
      $q->is_page              = false;
    }

    // When orderby is set, WordPress shows posts. Get around that here.
    if ( $q->is_home() && 'page' === get_option( 'show_on_front' ) && absint( get_option( 'page_on_front' ) ) === absint( get_option( 'udesign_designs_page_id', 0 ) ) ) {
      $_query = wp_parse_args( $q->query );
      if ( empty( $_query ) || ! array_diff( array_keys( $_query ), array( 'preview', 'page', 'paged', 'cpage', 'orderby' ) ) ) {
        $q->is_page = true;
        $q->is_home = false;
        $q->set( 'page_id', (int) get_option( 'page_on_front' ) );
        $q->set( 'post_type', 'design' );
      }
    }

    // Fix design feeds
    if ( $q->is_feed() && $q->is_post_type_archive( 'design' ) ) {
      $q->is_comment_feed = false;
    }

    // Special check for shops with the designs archive on front
    if ( $q->is_page() && 'page' === get_option( 'show_on_front' ) && absint( $q->get( 'page_id' ) ) === absint( get_option( 'udesign_designs_page_id', 0 ) ) ) {

      // This is a front-page shop
      $q->set( 'post_type', 'design' );
      $q->set( 'page_id', '' );

      if ( isset( $q->query['paged'] ) ) {
        $q->set( 'paged', $q->query['paged'] );
      }

      // Define a variable so we know this is the front page shop later on
      define( 'DESIGNS_IS_ON_FRONT', true );

      // Get the actual WP page to avoid errors and let us use is_front_page()
      // This is hacky but works. Awaiting https://core.trac.wordpress.org/ticket/21096
      global $wp_post_types;

      $designs_page  = get_post( vam_udesign_get_page_id( 'designs' ) );

      $wp_post_types['design']->ID          = $designs_page->ID;
      $wp_post_types['design']->post_title  = $designs_page->post_title;
      $wp_post_types['design']->post_name   = $designs_page->post_name;
      $wp_post_types['design']->post_type   = $designs_page->post_type;
      $wp_post_types['design']->ancestors   = get_ancestors( $designs_page->ID, $designs_page->post_type );

      // Fix conditional Functions like is_front_page
      $q->is_singular          = false;
      $q->is_post_type_archive = true;
      $q->is_archive           = true;
      $q->is_page              = true;

      // Remove post type archive name from front page title tag
      add_filter( 'post_type_archive_title', '__return_empty_string', 5 );

      // Fix WP SEO
      if ( class_exists( 'WPSEO_Meta' ) ) {
        add_filter( 'wpseo_metadesc', array( $this, 'wpseo_metadesc' ) );
        add_filter( 'wpseo_metakey', array( $this, 'wpseo_metakey' ) );
      }

    // Only apply to design categories, the design post archive, the shop page, design tags, and design attribute taxonomies
    } elseif ( ! $q->is_post_type_archive( 'design' ) && ! $q->is_tax( get_object_taxonomies( 'design' ) ) ) {
      return;
    }

    $this->design_query( $q );

    // And remove the pre_get_posts hook
    $this->remove_design_query();
  }

  /**
   * WP SEO meta description.
   *
   * Hooked into wpseo_ hook already, so no need for function_exist.
   *
   * @access public
   * @return string
   */
  public function wpseo_metadesc() {
    return WPSEO_Meta::get_value( 'metadesc', vam_udesign_get_page_id( 'designs' ) );
  }

  /**
   * WP SEO meta key.
   *
   * Hooked into wpseo_ hook already, so no need for function_exist.
   *
   * @access public
   * @return string
   */
  public function wpseo_metakey() {
    return WPSEO_Meta::get_value( 'metakey', vam_udesign_get_page_id( 'designs' ) );
  }

  /**
   * Query the designs, applying sorting/ordering etc. This applies to the main wordpress loop.
   *
   * @param mixed $q
   */
  public function design_query( $q ) {
    // Ordering query vars
    $ordering  = $this->get_catalog_ordering_args();
    $q->set( 'orderby', $ordering['orderby'] );
    $q->set( 'order', $ordering['order'] );
    if ( isset( $ordering['meta_key'] ) ) {
      $q->set( 'meta_key', $ordering['meta_key'] );
    }

    // Query vars that affect posts shown
    $q->set( 'meta_query', $this->get_meta_query( $q->get( 'meta_query' ) ) );
    $q->set( 'posts_per_page', $q->get( 'posts_per_page' ) ? $q->get( 'posts_per_page' ) : apply_filters( 'loop_shop_per_page', get_option( 'posts_per_page' ) ) );
    $q->set( 'udesign_query', 'design_query' );
    $q->set( 'post__in', array_unique( (array) apply_filters( 'loop_shop_post_in', array() ) ) );

    do_action( 'vam_udesign_design_query', $q, $this );
  }


  /**
   * Remove the query.
   */
  public function remove_design_query() {
    remove_action( 'pre_get_posts', array( $this, 'pre_get_posts' ) );
  }

  /**
   * Returns an array of arguments for ordering designs based on the selected values.
   *
   * @access public
   * @return array
   */
  public function get_catalog_ordering_args( $orderby = '', $order = '' ) {
    // Get ordering from query string unless defined
    if ( ! $orderby ) {
      $orderby_value = isset( $_GET['orderby'] ) ? wc_clean( $_GET['orderby'] ) : apply_filters( 'vam_udesign_default_catalog_orderby', 'title' );

      // Get order + orderby args from string
      $orderby_value = explode( '-', $orderby_value );
      $orderby       = esc_attr( $orderby_value[0] );
      $order         = ! empty( $orderby_value[1] ) ? $orderby_value[1] : $order;
    }

    $orderby = strtolower( $orderby );
    $order   = strtoupper( $order );
    $args    = array();

    // default - title
    $args['orderby']  = 'title';
    $args['order']    = $order == 'DESC' ? 'DESC' : 'ASC';
    $args['meta_key'] = '';

    switch ( $orderby ) {
      case 'rand' :
        $args['orderby']  = 'rand';
      break;
      case 'date' :
        $args['orderby']  = 'date ID';
        $args['order']    = $order == 'ASC' ? 'ASC' : 'DESC';
      break;
      case 'title' :
        $args['orderby']  = 'title';
        $args['order']    = $order == 'DESC' ? 'DESC' : 'ASC';
      break;
    }

    return apply_filters( 'vam_udesign_get_catalog_ordering_args', $args );
  }

  /**
   * Appends meta queries to an array.
   *
   * @param  array $meta_query
   * @return array
   */
  public function get_meta_query( $meta_query = array() ) {
    if ( ! is_array( $meta_query ) ) {
      $meta_query = array();
    }

    return array_filter( apply_filters( 'vam_udesign_design_query_meta_query', $meta_query, $this ) );
  }

  /**
   * Get the meta query which was used by the main query.
   * @return array
   */
  public static function get_main_meta_query() {
    global $wp_the_query;

    $args       = $wp_the_query->query_vars;
    $meta_query = isset( $args['meta_query'] ) ? $args['meta_query'] : array();

    return $meta_query;
  }

  /**
   * Based on WP_Query::parse_search
   */
  public static function get_main_search_query_sql() {
    global $wp_the_query, $wpdb;

    $args         = $wp_the_query->query_vars;
    $search_terms = isset( $args['search_terms'] ) ? $args['search_terms'] : array();
    $sql          = array();

    foreach ( $search_terms as $term ) {
      // Terms prefixed with '-' should be excluded.
      $include = '-' !== substr( $term, 0, 1 );

      if ( $include ) {
        $like_op  = 'LIKE';
        $andor_op = 'OR';
      } else {
        $like_op  = 'NOT LIKE';
        $andor_op = 'AND';
        $term     = substr( $term, 1 );
      }

      $like  = '%' . $wpdb->esc_like( $term ) . '%';
      $sql[] = $wpdb->prepare( "(($wpdb->posts.post_title $like_op %s) $andor_op ($wpdb->posts.post_excerpt $like_op %s) $andor_op ($wpdb->posts.post_content $like_op %s))", $like, $like, $like );
    }

    if ( ! empty( $sql ) && ! is_user_logged_in() ) {
      $sql[] = "($wpdb->posts.post_password = '')";
    }

    return implode( ' AND ', $sql );
  }

}
