<?php

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_Shortcodes class
 *
 * @class       VAM_UDesign_Shortcodes
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
class VAM_UDesign_Shortcodes {

  /**
   * Init shortcodes.
   */
  public static function init() {
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-shortcode-designer.php';

    $shortcodes = array(
      'udesign_designer'     => __CLASS__ . '::udesign_designer',
      'designs'              => __CLASS__ . '::designs',
      'recent_designs'       => __CLASS__ . '::recent_designs',
      'best_selling_designs' => __CLASS__ . '::best_selling_designs',
      'featured_designs'     => __CLASS__ . '::featured_designs',
    );

    foreach ( $shortcodes as $shortcode => $function ) {
      add_shortcode( apply_filters( "{$shortcode}_shortcode_tag", $shortcode ), $function );
    }
  }

  /**
   * Shortcode Wrapper.
   *
   * @param string[] $function
   * @param array $atts (default: array())
   * @return string
   */
  public static function shortcode_wrapper(
    $function,
    $atts    = array(),
    $wrapper = array(
      'class'  => 'udesign woocommerce',
      'before' => null,
      'after'  => null
    )
  ) {
    ob_start();

    echo empty( $wrapper['before'] ) ? '<div class="' . esc_attr( $wrapper['class'] ) . '">' : $wrapper['before'];
    call_user_func( $function, $atts );
    echo empty( $wrapper['after'] ) ? '</div>' : $wrapper['after'];

    return ob_get_clean();
  }

  /**
   * Designer page shortcode.
   *
   * @param mixed $atts
   * @return string
   */
  public static function udesign_designer( $atts ) {
    $atts = shortcode_atts( array(), $atts, 'udesign_designer' );

    return self::shortcode_wrapper( array( 'VAM_UDesign_Shortcode_Designer', 'output' ), $atts );
  }

  /**
   * Loop over found designs.
   * @param  array $query_args
   * @param  array $atts
   * @param  string $loop_name
   * @return string
   */
  private static function design_loop( $query_args, $atts, $loop_name ) {
    global $udesign_loop;

    $designs                 = new WP_Query( apply_filters( 'vam_udesign_shortcode_designs_query', $query_args, $atts, $loop_name ) );
    $columns                 = absint( $atts['columns'] );
    $udesign_loop['columns'] = $columns;
    $udesign_loop['name']    = $loop_name;

    ob_start();

    if ( $designs->have_posts() ) {
      ?>

      <?php do_action( "vam_udesign_shortcode_before_{$loop_name}_loop" ); ?>

      <?php vam_udesign_design_loop_start(); ?>

        <?php while ( $designs->have_posts() ) : $designs->the_post(); ?>

          <?php vam_udesign_get_template_part( 'content', 'design' ); ?>

        <?php endwhile; // end of the loop. ?>

      <?php vam_udesign_design_loop_end(); ?>

      <?php do_action( "vam_udesign_shortcode_after_{$loop_name}_loop" ); ?>

      <?php
    } else {
      do_action( "vam_udesign_shortcode_{$loop_name}_loop_no_results" );
    }

    vam_udesign_reset_loop();
    wp_reset_postdata();

    return '<div class="woocommerce udesign columns-' . $columns . '">' . ob_get_clean() . '</div>';
  }

  /**
   * List multiple designs shortcode.
   *
   * @param array $atts
   * @return string
   */
  public static function designs( $atts ) {
    $atts = shortcode_atts( array(
      'columns' => '4',
      'orderby' => 'title',
      'order'   => 'asc',
      'ids'     => '',
    ), $atts, 'designs' );

    $query_args = array(
      'post_type'           => 'design',
      'post_status'         => 'publish',
      'ignore_sticky_posts' => 1,
      'orderby'             => $atts['orderby'],
      'order'               => $atts['order'],
      'posts_per_page'      => -1
    );

    if ( ! empty( $atts['ids'] ) ) {
      $query_args['post__in'] = array_map( 'trim', explode( ',', $atts['ids'] ) );
    }

    return self::design_loop( $query_args, $atts, 'designs' );
  }

  /**
   * Recent designs shortcode.
   *
   * @param array $atts
   * @return string
   */
  public static function recent_designs( $atts ) {
    $atts = shortcode_atts( array(
      'per_page' => '12',
      'columns'  => '4',
      'orderby'  => 'date',
      'order'    => 'desc',
      'category' => '',  // Slugs
      'operator' => 'IN', // Possible values are 'IN', 'NOT IN', 'AND'.
    ), $atts, 'recent_designs' );

    $query_args = array(
      'post_type'           => 'design',
      'post_status'         => 'publish',
      'ignore_sticky_posts' => 1,
      'posts_per_page'      => $atts['per_page'],
      'orderby'             => $atts['orderby'],
      'order'               => $atts['order'],
    );

    $query_args = self::add_category_args( $query_args, $atts['category'], $atts['operator'] );

    return self::design_loop( $query_args, $atts, 'recent_designs' );
  }

  /**
   * List best selling designs on sale.
   *
   * @param array $atts
   * @return string
   */
  public static function best_selling_designs( $atts ) {
    $atts = shortcode_atts( array(
      'per_page' => '12',
      'columns'  => '4',
      'category' => '',  // Slugs
      'operator' => 'IN' // Possible values are 'IN', 'NOT IN', 'AND'.
    ), $atts, 'best_selling_designs' );

    $query_args = array(
      'post_type'           => 'design',
      'post_status'         => 'publish',
      'ignore_sticky_posts' => 1,
      'posts_per_page'      => $atts['per_page'],
      'meta_key'            => 'total_sales',
      'orderby'             => 'meta_value_num',
    );

    $query_args = self::add_category_args( $query_args, $atts['category'], $atts['operator'] );

    return self::design_loop( $query_args, $atts, 'best_selling_designs' );
  }

  /**
   * Output featured designs.
   *
   * @param array $atts
   * @return string
   */
  public static function featured_designs( $atts ) {
    $atts = shortcode_atts( array(
      'per_page' => '12',
      'columns'  => '4',
      'orderby'  => 'date',
      'order'    => 'desc',
      'category' => '',  // Slugs
      'operator' => 'IN', // Possible values are 'IN', 'NOT IN', 'AND'.
    ), $atts, 'featured_designs' );

    $meta_query[] = array(
      'key'     => '_design_featured',
      'value'   => 'yes',
      'compare' => '='
    );

    $query_args = array(
      'post_type'           => 'design',
      'post_status'         => 'publish',
      'ignore_sticky_posts' => 1,
      'posts_per_page'      => $atts['per_page'],
      'orderby'             => $atts['orderby'],
      'order'               => $atts['order'],
      'meta_query'          => $meta_query,
    );

    $query_args = self::add_category_args( $query_args, $atts['category'], $atts['operator'] );

    return self::design_loop( $query_args, $atts, 'featured_designs' );
  }

  /**
   * Adds a tax_query index to the query to filter by design category.
   *
   * @param array $args
   * @param string $category
   * @param string $operator
   * @return array;
   * @access private
   */
  private static function add_category_args( $args, $category, $operator ) {
    if ( ! empty( $category ) ) {
      $args['tax_query'] = array(
        array(
          'taxonomy' => 'design_cat',
          'terms'    => array_map( 'sanitize_title', explode( ',', $category ) ),
          'field'    => 'slug',
          'operator' => $operator
        )
      );
    }

    return $args;
  }
}
