<?php
/**
 * VAM_UDesign_WC_Compat class
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * Functions for WC core back-compatibility.
 *
 * @class    VAM_UDesign_WC_Compat
 * @version  1.0.0
 */
class VAM_UDesign_WC_Compat {

  /**
   * Compatibility function to get the version of the currently installed WooCommerce
   *
   * @since 1.0.0
   * @return string woocommerce version number or null
   */
  public static function get_wc_version() {
    if ( defined( 'WC_VERSION' ) && WC_VERSION )
      return WC_VERSION;
    if ( defined( 'WOOCOMMERCE_VERSION' ) && WOOCOMMERCE_VERSION )
      return WOOCOMMERCE_VERSION;

    return null;
  }

  /**
   * Returns true if the installed version of WooCommerce is greater than $version.
   *
   * @since  1.0.0
   *
   * @param  string  $version the version to compare
   * @return boolean true if the installed version of WooCommerce is > $version
   */
  public static function is_wc_version_gt( $version ) {
    return self::get_wc_version() && version_compare( self::get_wc_version(), $version, '>' );
  }

  /**
   * Returns true if the installed version of WooCommerce is greater than or equal to $version.
   *
   * @since  1.0.0
   *
   * @param  string  $version the version to compare
   * @return boolean true if the installed version of WooCommerce is >= $version
   */
  public static function is_wc_version_gte( $version ) {
    return self::get_wc_version() && version_compare( self::get_wc_version(), $version, '>=' );
  }

  /**
   * Compatibility function for outputting a woocommerce attribute label
   *
   * @since 1.0.0
   * @param string $label the label to display
   * @return string the label to display
   */
  public static function wc_attribute_label( $label ) {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return wc_attribute_label( $label );
    } else {
      global $woocommerce;
      return $woocommerce->attribute_label( $label );
    }
  }

  public static function wc_attribute_taxonomy_name( $name ) {
    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return wc_attribute_taxonomy_name( $name );
    } else {
      global $woocommerce;
      return $woocommerce->attribute_taxonomy_name( $name );
    }
  }

  public static function wc_get_attribute_taxonomies() {
    global $woocommerce;
    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return wc_get_attribute_taxonomies();
    } else {
      return $woocommerce->get_attribute_taxonomies();
    }
  }

  public static function wc_placeholder_img_src() {
    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return wc_placeholder_img_src();
    } else {
      return woocommerce_placeholder_img_src();
    }
  }

  /**
   * Compatibility function to add and store a notice
   *
   * @since 1.0.0
   * @param string $message The text to display in the notice.
   * @param string $notice_type The singular name of the notice type - either error, success or notice. [optional]
   */
  public static function wc_add_notice( $message, $notice_type = 'success' ) {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      wc_add_notice( $message, $notice_type );
    } else {
      global $woocommerce;

      if ( 'error' == $notice_type ) {
        $woocommerce->add_error( $message );
      } else {
        $woocommerce->add_message( $message );
      }
    }
  }

  /**
   * Prints messages and errors which are stored in the session, then clears them.
   *
   * @since 1.0.0
   */
  public static function wc_print_notices() {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      wc_print_notices();
    } else {
      global $woocommerce;
      $woocommerce->show_messages();
    }
  }

  /**
   * Compatibility function to queue some JavaScript code to be output in the footer.
   *
   * @since 1.0.0
   * @param string $code javascript
   */
  public static function wc_enqueue_js( $code ) {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      wc_enqueue_js( $code );
    } else {
      global $woocommerce;
      $woocommerce->add_inline_js( $code );
    }
  }

  /**
   * Sets WooCommerce messages
   *
   * @since 1.0.0
   */
  public static function set_messages() {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      // no-op in WC 2.1+
    } else {
      global $woocommerce;
      $woocommerce->set_messages();
    }
  }

  /**
   * Returns a new instance of the woocommerce logger
   *
   * @since 1.0.0
   * @return object logger
   */
  public static function new_wc_logger() {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return new WC_Logger();
    } else {
      global $woocommerce;
      return $woocommerce->logger();
    }
  }

  /**
   * Format decimal numbers ready for DB storage
   *
   * Sanitize, remove locale formatting, and optionally round + trim off zeros
   *
   * @since 1.0.0
   * @param  float|string $number Expects either a float or a string with a decimal separator only (no thousands)
   * @param  mixed $dp number of decimal points to use, blank to use woocommerce_price_num_decimals, or false to avoid all rounding.
   * @param  boolean $trim_zeros from end of string
   * @return string
   */
  public static function wc_format_decimal( $number, $dp = false, $trim_zeros = false ) {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return wc_format_decimal( $number, $dp, $trim_zeros );
    } else {
      return woocommerce_format_total( $number );
    }
  }

  /**
   * Get the count of notices added, either for all notices (default) or for one particular notice type specified
   * by $notice_type.
   *
   * @since 1.0.0
   * @param string $notice_type The name of the notice type - either error, success or notice. [optional]
   * @return int the notice count
   */
  public static function wc_notice_count( $notice_type = '' ) {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return wc_notice_count( $notice_type );
    } else {
      global $woocommerce;

      if ( 'error' == $notice_type ) {
        return $woocommerce->error_count();
      } else {
        return $woocommerce->message_count();
      }
    }
  }

  /**
   * Compatibility function to use the new WC_Admin_Meta_Boxes class for the save_errors() function
   *
   * @since 1.0.0
   * @return old save_errors function or new class
   */
  public static function save_errors() {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      WC_Admin_Meta_Boxes::save_errors();
    } else {
      woocommerce_meta_boxes_save_errors();
    }
  }

  /**
   * Returns the WooCommerce instance
   *
   * @since 1.0.0
   * @return WooCommerce woocommerce instance
   */
  public static function WC() {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return WC();
    } else {
      global $woocommerce;
      return $woocommerce;
    }
  }

  /**
   * Returns true if the WooCommerce plugin is loaded
   *
   * @since 1.0.0
   * @return boolean true if WooCommerce is loaded
   */
  public static function is_wc_loaded() {

    if ( self::is_wc_version_gt( '2.0.20' ) ) {
      return class_exists( 'WooCommerce' );
    } else {
      return class_exists( 'Woocommerce' );
    }
  }

  /**
   * Get the WC Product instance for a given product ID or post.
   *
   * get_product() is soft-deprecated in WC 2.2
   *
   * @since 1.0.0
   *
   * @param  bool|int|string|WP_Post  $the_product
   * @param  array                    $args
   * @return WC_Product
   */
  public static function wc_get_product( $the_product = false, $args = array() ) {
    if ( self::is_wc_version_gte( '2.2' ) ) {
      return wc_get_product( $the_product, $args );
    } else {

      return get_product( $the_product, $args );
    }
  }

  /**
   * Get all product cats for a product by ID, including hierarchy.
   *
   * @since  1.0.0
   *
   * @param  int  $product_id
   * @return array
   */
  public static function wc_get_product_cat_ids( $product_id ) {
    if ( self::is_wc_version_gte( '2.5' ) ) {
      $product_cats = wc_get_product_cat_ids( $product_id );
    } else {

      $product_cats = wp_get_post_terms( $product_id, 'product_cat', array( "fields" => "ids" ) );

      foreach ( $product_cats as $product_cat ) {
        $product_cats = array_merge( $product_cats, get_ancestors( $product_cat, 'product_cat' ) );
      }
    }

    return $product_cats;
  }

  /**
   * Wrapper for wp_get_post_terms which supports ordering by parent.
   *
   * @since  1.0.0
   *
   * @param  int     $product_id
   * @param  string  $taxonomy
   * @param  array   $args
   * @return array
   */
  public static function wc_get_product_terms( $product_id, $attribute_name, $args ) {
    if ( self::is_wc_version_gte( '2.3' ) ) {
      return wc_get_product_terms( $product_id, $attribute_name, $args );
    } else {

      $orderby = wc_attribute_orderby( sanitize_title( $attribute_name ) );

      switch ( $orderby ) {
        case 'name' :
          $args = array( 'orderby' => 'name', 'hide_empty' => false, 'menu_order' => false );
        break;
        case 'id' :
          $args = array( 'orderby' => 'id', 'order' => 'ASC', 'menu_order' => false );
        break;
        case 'menu_order' :
          $args = array( 'menu_order' => 'ASC' );
        break;
      }

      $terms = get_terms( sanitize_title( $attribute_name ), $args );

      return $terms;
    }
  }

  /**
   * Get rounding precision.
   *
   * @since  1.0.0
   *
   * @return int
   */
  public static function wc_get_rounding_precision( $price_decimals = false ) {
    if ( false === $price_decimals ) {
      $price_decimals = wc_get_price_decimals();
    }
    return absint( $price_decimals ) + 2;
  }

  /**
   * Return the number of decimals after the decimal point.
   *
   * @since  1.0.0
   *
   * @return int
   */
  public static function wc_get_price_decimals() {
    if ( self::is_wc_version_gte( '2.3' ) ) {
      return wc_get_price_decimals();
    } else {
      return absint( get_option( 'woocommerce_price_num_decimals', 2 ) );
    }
  }

  /**
   * Output a list of variation attributes for use in the cart forms.
   *
   * @since 1.0.0
   *
   * @param array  $args
   */
  public static function wc_dropdown_variation_attribute_options( $args = array() ) {
    return wc_dropdown_variation_attribute_options( $args );
  }

  /**
   * Display a WooCommerce help tip.
   *
   * @since  1.0.0
   *
   * @param  string  $tip
   * @return string
   */
  public static function wc_help_tip( $tip ) {

    if ( self::is_wc_version_gte( '2.5' ) ) {
      return wc_help_tip( $tip );
    } else {
      return '<img class="help_tip woocommerce-help-tip" data-tip="' . $tip . '" src="' . WC()->plugin_url() . '/assets/images/help.png" />';
    }
  }

  /**
   * Back-compat wrapper for 'wc_variation_attribute_name'.
   *
   * @since  1.0.0
   *
   * @param  string  $attribute_name
   * @return string
   */
  public static function wc_variation_attribute_name( $attribute_name ) {
    if ( self::is_wc_version_gte( '2.6' ) ) {
      return wc_variation_attribute_name( $attribute_name );
    } else {
      return 'attribute_' . sanitize_title( $attribute_name );
    }
  }

  /**
   * Back-compat wrapper for 'get_parent_id'.
   *
   * @since  1.0.0
   *
   * @param  WC_Product  $product
   * @return mixed
   */
  public static function get_parent_id( $product ) {
    if ( self::is_wc_version_gte( '3.0' ) ) {
      return $product->get_parent_id();
    } else {
      return $product->is_type( 'variation' ) ? absint( $product->id ) : 0;
    }
  }

  /**
   * Back-compat wrapper for 'get_id'.
   *
   * @since  1.0.0
   *
   * @param  WC_Data $data
   * @return mixed
   */
  public static function get_id( $data ) {
    if ( is_array( $data ) && isset( $obj['id'] ) ) {
      return $obj['id'];
    }

    if ( self::is_wc_version_gte( '3.0' ) ) {
      return $data->get_id();
    } else {
      return absint( $data->id );
    }
  }

  /**
   * Back-compat wrapper for 'get_id'.
   *
   * @since  1.0.0
   *
   * @param  WC_Product  $product
   * @return mixed
   */
  public static function get_product_id( $product ) {
    if ( self::is_wc_version_gte( '3.0' ) ) {
      return $product->get_id();
    } else {
      return $product->is_type( 'variation' ) ? absint( $product->variation_id ) : absint( $product->id );
    }
  }

  /**
   * Back-compat wrapper for getting CRUD object props directly.
   *
   * @since  1.0.0
   *
   * @param  object  $obj
   * @param  string  $name
   * @param  string  $context
   * @return mixed
   */
  public static function get_prop( $obj, $name, $context = 'view' ) {
    if ( is_array( $obj ) && isset( $obj[$name] ) ) {
      return $obj[$name];
    }

    $value = null;
    if ( self::is_wc_version_gte( '3.0' ) ) {
      $get_fn = 'get_' . $name;
      $value = is_callable( array( $obj, $get_fn ) ) ? $obj->$get_fn( $context ) : null;
    } else {
      if ( 'name' === $name ) {
        $value = isset( $obj->post->post_title ) ? $obj->post->post_title : null;
      } else if ( 'status' === $name ) {
        $value = isset( $obj->post->post_status ) ? $obj->post->post_status : null;
      } elseif ( 'short_description' === $name ) {
        $value = isset( $obj->post->post_excerpt ) ? $obj->post->post_excerpt : null;
      } else {
        $value = $obj->$name;
      }
    }

    return $value;
  }

  /**
   * Back-compat wrapper for setting CRUD object props directly.
   *
   * @since  1.0.0
   *
   * @param  WC_Product  $product
   * @param  string      $name
   * @param  mixed       $value
   * @return void
   */
  public static function set_prop( $obj, $name, $value ) {
    if ( is_array( $obj ) ) {
      $obj[$name] = $value;
      return;
    }

    if ( self::is_wc_version_gte( '3.0' ) ) {
      $set_fn = 'set_' . $name;
      $obj->$set_fn( $value );
    } else {
      $obj->$name = $value;
    }
  }

  /**
   * Back-compat wrapper for 'wc_get_formatted_variation'.
   *
   * @since  1.0.0
   *
   * @param  WC_Product_Variation  $variation
   * @param  boolean               $flat
   * @return string
   */
  public static function wc_get_formatted_variation( $variation, $flat ) {
    if ( self::is_wc_version_gte( '3.0' ) ) {
      return wc_get_formatted_variation( $variation, $flat );
    } elseif ( self::is_wc_version_gte( '2.5' ) ) {
      return $variation->get_formatted_variation_attributes( $flat );
    } else {
      return wc_get_formatted_variation( $variation->get_variation_attributes(), $flat );
    }
  }

  /**
   * Get prefix for use with wp_cache_set. Allows all cache in a group to be invalidated at once..
   *
   * @since  1.0.0
   *
   * @param  string  $group
   * @return string
   */
  public static function wc_cache_helper_get_cache_prefix( $group ) {
    if ( self::is_wc_version_gte( '2.5' ) ) {
      return WC_Cache_Helper::get_cache_prefix( $group );
    } else {
      // Get cache key - uses cache key wc_orders_cache_prefix to invalidate when needed
      $prefix = wp_cache_get( 'wc_' . $group . '_cache_prefix', $group );

      if ( false === $prefix ) {
        $prefix = 1;
        wp_cache_set( 'wc_' . $group . '_cache_prefix', $prefix, $group );
      }

      return 'wc_cache_' . $prefix . '_';
    }
  }

  /**
   * Increment group cache prefix (invalidates cache).
   *
   * @since  1.0.0
   *
   * @param  string  $group
   */
  public static function wc_cache_helper_incr_cache_prefix( $group ) {
    if ( self::is_wc_version_gte( '2.5' ) ) {
      WC_Cache_Helper::incr_cache_prefix( $group );
    } else {
      wp_cache_incr( 'wc_' . $group . '_cache_prefix', 1, $group );
    }
  }

  /**
   * Get a product attribute ID by name.
   *
   * @since  1.0.0
   * @param string $name Attribute name.
   * @return int
   */
  public static function wc_attribute_taxonomy_id_by_name( $name ) {
    if ( self::is_wc_version_gte( '2.6' ) ) {
      return wc_attribute_taxonomy_id_by_name( $name );
    }

    $name       = str_replace( 'pa_', '', $name );
    $taxonomies = wp_list_pluck( wc_get_attribute_taxonomies(), 'attribute_id', 'attribute_name' );
    return isset( $taxonomies[ $name ] ) ? (int) $taxonomies[ $name ] : 0;
  }

  /**
   * Back-compat wrapper for 'get_variation_default_attributes'.
   *
   * @since  1.0.0
   *
   * @param  WC_Product $product
   * @return mixed
   */
  public static function get_default_attributes( $product ) {
    if ( self::is_wc_version_gte( '3.0' ) ) {
      return $product->get_default_attributes();
    } else {
      return $product->is_type( 'variable' ) ? $product->get_variation_default_attributes() : '';
    }
  }

}
