<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
final class VAM_UDesign {
  /**
   * The single instance of the class.
   *
   * @var VAM_UDesign
   * @since 1.0.0
   */
  protected static $_instance = null;

  /**
   * The loader that's responsible for maintaining and registering all hooks that power
   * the plugin.
   *
   * @since    1.0.0
   * @access   protected
   * @var      VAM_UDesign_Loader    $loader    Maintains and registers all hooks for the plugin.
   */
  protected $loader;

  /**
   * The unique identifier of this plugin.
   *
   * @since    1.0.0
   * @access   protected
   * @var      string    $plugin_name    The string used to uniquely identify this plugin.
   */
  protected $plugin_name;

  /**
   * The current version of the plugin.
   *
   * @since    1.0.0
   * @access   protected
   * @var      string    $version    The current version of the plugin.
   */
  protected $version;

  /**
   * VAM_UDesign minimum version required.
   *
   * @since    1.0.0
   * @access   protected
   * @var      string    $version    VAM_UDesign minimum version required.
   */
  protected $required;

  /**
   * Query instance.
   *
   * @since    1.0.0
   * @access   public
   * @var      VAM_UDesign_Query
   */
  public $query = null;

  /**
   * Main VAM_UDesign Instance.
   *
   * Ensures only one instance of VAM_UDesign is loaded or can be loaded.
   *
   * @since 1.0.0
   * @static
   * @see WC()
   * @return VAM_UDesign - Main instance.
   */
  public static function instance() {
    if ( is_null( self::$_instance ) ) {
      self::$_instance = new self();
    }

    return self::$_instance;
  }

  /**
   * Cloning is forbidden.
   * @since 1.0.0
   */
  public function __clone() {
    _doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'vam-udesign' ), '1.0.0' );
  }

  /**
   * Unserializing instances of this class is forbidden.
   * @since 1.0.0
   */
  public function __wakeup() {
    _doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'vam-udesign' ), '1.0.0' );
  }

  /**
   * Define the core functionality of the plugin.
   *
   * Set the plugin name and the plugin version that can be used throughout the plugin.
   * Load the dependencies, define the locale, and set the hooks for the admin area and
   * the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function __construct() {
    $this->plugin_name = 'vam-udesign';
    $this->version = '1.2.3';
    $this->required = '2.7.0';

    $this->define_constants();
    $this->load_dependencies();
    $this->set_locale();
    $this->define_admin_hooks();
    $this->define_public_hooks();
    $this->define_update_hooks();
  }

  /**
   * Define Constants.
   */
  private function define_constants() {
    /* Designer information */
    $this->define( 'UDESIGN_VERSION', '1.0.5' );
    $this->define( 'UDESIGN_URL', '//designer.vectorartmedia.com' );
    $this->define( 'UDESIGN_DEBUG', false );
    $this->define( 'UDESIGN_DEBUG_URL', 'http://127.0.0.1:9001' );
    $this->define( 'UDESIGN_PRODUCTION_URL', UDESIGN_URL . '/designer/' . UDESIGN_VERSION );

    /* Plugin information */
    $this->define( 'VAM_UDESIGN_VERSION', $this->version );
    $this->define( 'VAM_UDESIGN_UPDATE_URL', UDESIGN_URL . '/integration/wordpress/vam-woocommerce-udesign/latest' );

    /* Plugin details */
    $VAM_UDESIGN_DETAILS = [
      'name' => 'uDesign',
      'author' => '<a href="https://vectorartmedia.com">VectorArt Media</a>',
      'requires' => '3.0',
      'tested' => '4.6.1',
      'homepage' => 'https://vectorartmedia.com',
      'sections' => [
        'description' => 'uDesign Product Designer integration for Wordpress',
      ]
    ];
  }

  /**
   * Define constant if not already set.
   *
   * @param  string $name
   * @param  string|bool $value
   */
  private function define( $name, $value ) {
    if ( ! defined( $name ) ) {
      define( $name, $value );
    }
  }

  /**
   * Load the required dependencies for this plugin.
   *
   * Include the following files that make up the plugin:
   *
   * - VAM_UDesign_Loader. Orchestrates the hooks of the plugin.
   * - VAM_UDesign_i18n. Defines internationalization functionality.
   * - VAM_UDesign_Admin. Defines all hooks for the admin area.
   * - VAM_UDesign_Public. Defines all hooks for the public side of the site.
   *
   * Create an instance of the loader which will be used to register the hooks
   * with WordPress.
   *
   * @since    1.0.0
   * @access   private
   */
  private function load_dependencies() {
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-wc-core-compatibility.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-zip-file.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/vam-udesign-core-functions.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/vam-udesign-product-functions.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-migrations.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-updater.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-loader.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-i18n.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-ajax.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-wc-swatch-term.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-design.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-product.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-query.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-post-types.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-shortcodes.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-media.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-cart.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-order.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-product-importer.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-vam-udesign-admin.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-vam-udesign-public.php';
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'api/vam-udesign-api.php';

    $this->loader = new VAM_UDesign_Loader();
    $this->query  = new VAM_UDesign_Query();
  }

  public static function on_init() {
    $image_size     = get_option( 'attribute_swatch', array() );
    $size           = array();
    $size['width']  = isset( $image_size['width'] ) && ! empty( $image_size['width'] ) ? $image_size['width'] : '26';
    $size['height'] = isset( $image_size['height'] ) && ! empty( $image_size['height'] ) ? $image_size['height'] : '26';
    $size['crop']   = isset( $image_size['crop'] ) ? $image_size['crop'] : array( 'center', 'center' );

    /* Add image sizes */
    add_image_size( 'attribute_swatch', $size['width'], $size['height'], $size['crop'] );
    add_image_size( 'designer_thumb', 180, 200, array( 'center', 'center' ) );
    add_image_size( 'designer_view', 650, 728, array( 'center', 'center' ) );
    add_image_size( 'designer_view_landscape', 250 );
    add_image_size( 'designer_view_portrait', 0, 280, false );

    /* define REST API constants */
    self::define_api_constants();

    /* run migrations */
    VAM_UDesign_Migrations::run();

    /* initialize shortcodes */
    VAM_UDesign_Shortcodes::init();

    /* initialize media */
    VAM_UDesign_Media::init();

    /* initialize orders */
    VAM_UDesign_Order::init();
  }

  public static function on_rest_api_init() {
    VAM_UDesign_REST_API::init();
  }

  public static function define_api_constants() {
    define( 'VAM_UDESIGN_API_HOST', 'https://demo.vectorartmedia.com' );
    define( 'VAM_UDESIGN_API_URI', '/wp-json/udesign/v1' );
    define( 'VAM_UDESIGN_API_URL', VAM_UDESIGN_API_HOST . VAM_UDESIGN_API_URI );

    define( 'VAM_UDESIGN_REQUIRE_AUTH', false );
    define( 'VAM_UDESIGN_API_USERNAME', '' );
    define( 'VAM_UDESIGN_API_PASSWORD', '' );
  }

  /**
   * Define the locale for this plugin for internationalization.
   *
   * Uses the VAM_UDesign_i18n class in order to set the domain and to register the hook
   * with WordPress.
   *
   * @since    1.0.0
   * @access   private
   */
  private function set_locale() {

    $plugin_i18n = new VAM_UDesign_i18n();

    $this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

  }

  /**
   * Register all of the hooks related to the admin area functionality
   * of the plugin.
   *
   * @since    1.0.0
   * @access   private
   */
  private function define_admin_hooks() {

    $plugin_admin = new VAM_UDesign_Admin( $this->plugin_name(), $this->version() );

    $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
    $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

  }

  /**
   * Register all of the hooks related to the public-facing functionality
   * of the plugin.
   *
   * @since    1.0.0
   * @access   private
   */
  private function define_public_hooks() {
    $plugin_public = new VAM_UDesign_Public( $this->plugin_name(), $this->version() );
  }

  /**
   * Register all of the hooks related to the plugin update functionality
   * of the plugin.
   *
   * @since    1.0.0
   * @access   private
   */
  private function define_update_hooks() {
    $plugin_update = new VAM_UDesign_Updater( $this->plugin_name, VAM_UDESIGN_PLUGIN_SLUG,
      $this->version, UDESIGN_URL, VAM_UDESIGN_UPDATE_URL );

    $this->loader->add_filter( 'http_request_host_is_external', $plugin_update, 'allowed_sources', 10, 3 );
    $this->loader->add_filter( 'pre_set_site_transient_update_plugins', $plugin_update, 'check_update', 10, 1 );
  }

  /**
   * Run the loader to execute all of the hooks with WordPress.
   *
   * @since    1.0.0
   */
  public function run() {
    $this->loader->add_action( 'init', __CLASS__,  'on_init' );
    $this->loader->add_action( 'rest_api_init', __CLASS__, 'on_rest_api_init' );
    $this->loader->run();
  }

  /**
   * The name of the plugin used to uniquely identify it within the context of
   * WordPress and to define internationalization functionality.
   *
   * @since     1.0.0
   * @return    string    The name of the plugin.
   */
  public function plugin_name() {
    return $this->plugin_name;
  }

  /**
   * The reference to the class that orchestrates the hooks with the plugin.
   *
   * @since     1.0.0
   * @return    VAM_UDesign_Loader    Orchestrates the hooks of the plugin.
   */
  public function loader() {
    return $this->loader;
  }

  /**
   * Retrieve the version number of the plugin.
   *
   * @since     1.0.0
   * @return    string    The version number of the plugin.
   */
  public function version() {
    return $this->version;
  }

  /**
   * Get the plugin url.
   * @return string
   */
  public function plugin_url( $path = '' ) {
    $url = untrailingslashit( plugins_url( '/', __DIR__ ) );
    if ( $path && is_string( $path ) ) {
      $url .= '/' . ltrim($path, '/');
    }

    return $url;
  }

  /**
   * Get the plugin path.
   * @return string
   */
  public function plugin_path() {
    return untrailingslashit( plugin_dir_path( __DIR__ ) );
  }

  /**
   * Get the template path.
   * @return string
   */
  public function template_path() {
    return 'udesign/';
  }

}

/**
 * Main instance of VAM_UDesign.
 *
 * Returns the main instance of WC to prevent the need to use globals.
 *
 * @since  1.0.0
 * @return VAM_UDesign
 */
function VAM_UDesign() {
  return VAM_UDesign::instance();
}
