<?php
/**
 * VAM_UDesign Design Functions
 *
 * Functions for design specific things.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * Main function for returning designs.
 *
 * @param mixed $the_design Post object or post ID of the design.
 * @param array $args (default: array()) Contains all arguments to be used to get this design.
 * @return WC_Product
 */
function vam_udesign_get_design( $the_design = false, $args = array() ) {
  try {
    if ( false === $the_design ) {
      $the_design = $GLOBALS['post'];
    } elseif ( is_numeric( $the_design ) ) {
      $the_design = get_post( $the_design );
    } elseif ( $the_design instanceof VAM_UDesign_Design ) {
      $the_design = get_post( $the_design->id );
    } elseif ( ! ( $the_design instanceof WP_Post ) ) {
      $the_design = false;
    }

    if ( ! $the_design ) {
      throw new Exception( 'Design object does not exist', 422 );
    }

    $classname = ( 'design' === $the_design->post_type ) ? 'VAM_UDesign_Design' : false;

    if ( ! $classname ) {
      throw new Exception( 'Missing classname', 422 );
    }

    if ( ! class_exists( $classname ) ) {
      $classname = 'VAM_UDesign_Design';
    }

    return new $classname( $the_design, $args );

  } catch ( Exception $e ) {
    return false;
  }
}

/**
 * Get all design cats for a design by ID, including hierarchy
 * @since  1.0.0
 * @param  int $design_id
 * @return array
 */
function vam_udesign_get_design_cat_ids( $design_id ) {
  $design_cats = wp_get_post_terms( $design_id, 'design_cat', array( 'fields' => 'ids' ) );

  foreach ( $design_cats as $design_cat ) {
    $design_cats = array_merge( $design_cats, get_ancestors( $design_cat, 'design_cat' ) );
  }

  return $design_cats;
}

/**
 * Gets data about an attachment, such as alt text and captions.
 * @since 1.0.0
 * @param object|bool $design
 * @return array
 */
function vam_udesign_get_design_attachment_props( $attachment_id, $design = false ) {
  $props = array(
    'title'   => '',
    'caption' => '',
    'url'     => '',
    'alt'     => '',
  );

  if ( $attachment_id ) {
    $attachment       = get_post( $attachment_id );
    $props['title']   = trim( strip_tags( $attachment->post_title ) );
    $props['caption'] = trim( strip_tags( $attachment->post_excerpt ) );
    $props['url']     = wp_get_attachment_url( $attachment_id );
    $props['alt']     = trim( strip_tags( get_post_meta( $attachment_id, '_wp_attachment_image_alt', true ) ) );

    // Alt text fallbacks
    $props['alt']     = empty( $props['alt'] ) ? $props['caption'] : $props['alt'];
    $props['alt']     = empty( $props['alt'] ) ? trim( strip_tags( $attachment->post_title ) ) : $props['alt'];
    $props['alt']     = empty( $props['alt'] ) && $design ? trim( strip_tags( get_the_title( $design->ID ) ) ) : $props['alt'];
  }

  return $props;
}

/**
 * Get a list of designs based on the parameter filters
 * @since  1.0.0
 * @param array $args (default: array()) Contains all arguments to be used to get the list of designs.
 * @return array
 */
function vam_udesign_get_designs( $args = array() ) {
  // Base query arguments
  $query_args = array(
    'fields'         => 'ids',
    'udesign_query'  => 'design_query',
    'orderby'        => 'title',
    'order'          => 'ASC',
    'post_type'      => 'design',
    'post_status'    => 'publish',
    'posts_per_page' => -1,
    'tax_query'      => array(),
    'meta_query'     => array()
  );

  // Post status
  if ( !empty( $args['post_status'] ) ) {
    $query_args['post_status'] = $args['post_status'];
  }

  // Post author
  if ( !empty( $args['post_author'] ) ) {
    $query_args['author'] = $args['post_author'];
  }

  // Fields
  if ( !empty( $args['fields'] ) ) {
    $query_args['fields'] = $args['fields'];
  }

  // Pagination
  if ( !empty( $args['posts_per_page'] ) ) {
    $query_args['posts_per_page'] = $args['posts_per_page'];
  }
  if ( !empty( $args['paged'] ) ) {
    $query_args['paged'] = $args['paged'];
  }

  // Date filter
  if ( !empty( $args['m'] ) ) {
    $query_args['m'] = $args['m'];
  }

  // Sorting
  if ( !empty( $args['orderby'] ) ) {
    $query_args['orderby'] = $args['orderby'];
  }
  if ( !empty( $args['order'] ) ) {
    $query_args['order'] = $args['order'];
  }

  // From category
  if ( !empty( $args['category_id'] ) ) {
    $query_args['tax_query'][] = array(
      'taxonomy' => 'design_cat',
      'terms' => $args['category_id'],
    );
  }

  // Search query
  if ( !empty( $args['search_query'] ) ) {
    $query_args['meta_query'][] = array(
      'key'     => '_vam_udesign_search_field',
      'value'   => strtolower( sanitize_text_field( $args['search_query'] ) ),
      'compare' => 'LIKE'
    );
  }

  // Search by remote id
  if ( !empty( $args['remote_id'] ) ) {
    $remote_id = $args['remote_id'];
    if ( !is_array( $remote_id ) ) {
      $remote_id = array( $remote_id );
    }

    $query_args['meta_query'][] = array(
      'key'     => '_vam_udesign_remote_id',
      'value'   => $remote_id,
      'compare' => 'in'
    );
  }

  // Get designs
  $designs = new WP_Query( $query_args );
  wp_reset_postdata();

  return $designs;
}

function vam_udesign_create_design( $args = array() ) {
  if ( empty( $args['name'] ) ) {
    return false;
  }
  if ( empty( $args['metadata'] ) ) {
    return false;
  }


  // insert post
  $design = array(
    'post_title' => $args['name'],
    'post_content' => '',
    'post_status' => 'publish',
    'post_type' => 'design'
  );

  if ( !empty( $args['post_author'] ) ) {
    $design['post_author'] = $args['post_author'];
  }

  if ( !empty( $args['post_status'] ) ) {
    $design['post_status'] = $args['post_status'];
  }

  $design_id = wp_insert_post( $design );
  if ( empty( $design_id ) ) {
    return false;
  }

  // set design metadata
  update_post_meta( $design_id, '_design_meta', $args['metadata'] );

  // set post thumb
  if ( !empty( $args['thumb'] ) ) {
    $thumb_id = vam_udesign_create_attachment( $args['thumb'], $design_id );
    if ( empty( $thumb_id ) ) {
      return false;
    }

    set_post_thumbnail( $design_id, $thumb_id );
  }

  // set categories
  if ( !empty( $args['category_ids'] ) ) {
    wp_set_object_terms( $design_id, $args['category_ids'], 'design_cat' );
  }

  // set tags
  if ( !empty( $args['tag_ids'] ) ) {
    wp_set_object_terms( $design_id, $args['tag_ids'], 'design_tag' );
  }

  // set search field
  if ( !empty( $args['search_field'] ) ) {
    update_post_meta( $design_id, '_vam_udesign_search_field', $args['search_field'] );
  }

  // set design background color
  if ( !empty( $args['background_color'] ) ) {
    update_post_meta( $design_id, '_design_background_color', $args['background_color'] );
  }

  // set design background color
  if ( !empty( $args['default_product_id'] ) ) {
    update_post_meta( $design_id, '_design_default_product_id', $args['default_product_id'] );
  }

  // set remote id
  if ( !empty( $args['remote_id'] ) ) {
    update_post_meta( $design_id, '_vam_udesign_remote_id', $args['remote_id'] );
  }

  // set design customization
  if ( !empty( $args['disable_customize'] ) ) {
    update_post_meta( $design_id, '_design_disable_customize', $args['disable_customize'] );
  }

  return $design_id;
}

function vam_udesign_get_post_thumbnail_size() {
  $additional_image_sizes = wp_get_additional_image_sizes();
  if ( !empty( $additional_image_sizes['post-thumbnail'] ) ) {
    return $additional_image_sizes['post-thumbnail'];
  }

  return array(
    'width' => 600,
    'height' => 600
  );
}

function vam_udesign_generate_design_thumbnail_for_size( $design_id, $path, $name, $size, $view, &$meta_array = false, $meta_key = false ) {
  $meta_val = ( !empty( $meta_key ) && !empty( $meta_array[$view][$meta_key] ) ) ? $meta_array[$view][$meta_key] : false;
  $to = empty( $meta_val ) ? false : vam_udesign_path_from_url( $meta_val );

  $result = vam_udesign_generate_thumbnail( $path, $name, intval( $size['width'] ), intval( $size['height'] ), $to );
  if ( empty( $result ) ) {
    return false;
  }

  return empty( $to ) ? $result : array( 'url' => $meta_val );
}

function vam_udesign_generate_design_thumbnails( $design_id, $views, $featured_image = array() ) {
  $thumbnails = array();
  $thumbnails_meta = get_post_meta( $design_id, '_design_thumbnails', true );
  $featured_image = false;

  foreach( $views as $index => &$view ) {
    $path = $view['tmp_name'];
    $view_id = absint( $view['name'] );
    $name = sprintf( 'design_%s_%d.png', $design_id, $view_id );
    $view['name'] = $name;

    if ( $index == 0  && empty( $featured_image ) ) {
      $size = vam_udesign_get_post_thumbnail_size();
      $featured_image = vam_udesign_generate_design_thumbnail_for_size( $design_id, $path, $name, $size, $view_id );
      if ( empty( $featured_image ) ) {
        return false;
      }
    }

    $size = vam_udesign_get_image_size( 'udesign_design_archive' );
    $archive_image = vam_udesign_generate_design_thumbnail_for_size( $design_id, $path, $name, $size, $view_id, $thumbnails_meta, 'archive' );
    if ( empty( $archive_image ) ) {
      return false;
    }

    $size = vam_udesign_get_image_size( 'udesign_design_single' );
    $single_image = vam_udesign_generate_design_thumbnail_for_size( $design_id, $path, $name, $size, $view_id, $thumbnails_meta, 'single' );
    if ( empty( $single_image ) ) {
      return false;
    }

    $size = vam_udesign_get_image_size( 'udesign_design_thumbnail' );
    $thumbnail_image = vam_udesign_generate_design_thumbnail_for_size( $design_id, $path, $name, $size, $view_id, $thumbnails_meta, 'thumbnail' );
    if ( empty( $thumbnail_image ) ) {
      return false;
    }

    $full_image = array();
    if ( !empty( $view['url'] ) ) {
      $full_image = array( 'url' => $view['url'] );
    } else {
      $full_image = empty( $thumbnails_meta[$view_id]['full_image'] ) ? false : $thumbnails_meta[$view_id]['full'];
      $full_image_path = empty( $full_image ) ? false : vam_udesign_path_from_url( $full_image );

      if ( empty( $full_image ) || empty( $full_image_path ) ) {
        $full_image = wp_handle_upload( $view, array( 'test_form' => false ) );
        if ( empty( $full_image ) || !empty( $full_image['error'] ) ) {
          return false;
        }
      } else {
        if ( empty( @move_uploaded_file( $path, $full_image_path ) ) ) {
          return false;
        }

        $full_image = array( 'url' => $full_image );
      }
    }

    if ( $index == 0 ) {
      update_post_meta( $design_id, '_design_archive_image', $archive_image['url'] );
      update_post_meta( $design_id, '_design_single_image', $single_image['url'] );
      update_post_meta( $design_id, '_design_thumbnail_image', $thumbnail_image['url'] );
      update_post_meta( $design_id, '_design_full_image', $full_image['url'] );
    }

    $thumbnails[$view_id] = array(
      'archive' => $archive_image['url'],
      'single' => $single_image['url'],
      'thumbnail' => $thumbnail_image['url'],
      'full' => $full_image['url']
    );
  }

  if ( empty( $featured_image ) ) {
    return false;
  }

  update_post_meta( $design_id, '_design_thumbnails', $thumbnails );

  $thumbnails['featured_image'] = $featured_image;
  return $thumbnails;
}
