<?php
/**
 * VAM_UDesign Page Functions
 *
 * Functions related to pages and menus.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * Retrieve page ids - used for designer, designs. returns -1 if no page is found.
 *
 * @param string $page
 * @return int
 */
function vam_udesign_get_page_id( $page ) {

  $page = apply_filters( 'vam_udesign_get_' . $page . '_page_id', get_option('udesign_' . $page . '_page_id' ) );

  return $page ? absint( $page ) : -1;
}

/**
 * Retrieve page permalink.
 *
 * @param string $page
 * @return string
 */
function vam_udesign_get_page_permalink( $page ) {
  $page_id   = vam_udesign_get_page_id( $page );
  $permalink = 0 < $page_id ? get_permalink( $page_id ) : get_home_url();
  return apply_filters( 'vam_udesign_get_' . $page . '_page_permalink', $permalink );
}

/**
 * Fix active class in nav for designs page.
 *
 * @param array $menu_items
 * @return array
 */
function vam_udesign_nav_menu_item_classes( $menu_items ) {
  if ( ! is_vam_udesign_page() ) {
    return $menu_items;
  }

  $designs_page   = (int) vam_udesign_get_page_id( 'designs' );
  $page_for_posts = (int) get_option( 'page_for_posts' );

  foreach ( (array) $menu_items as $key => $menu_item ) {
    $classes = (array) $menu_item->classes;

    // Unset active class for blog page
    if ( $page_for_posts == $menu_item->object_id ) {
      $menu_items[$key]->current = false;

      if ( in_array( 'current_page_parent', $classes ) ) {
        unset( $classes[ array_search('current_page_parent', $classes) ] );
      }

      if ( in_array( 'current-menu-item', $classes ) ) {
        unset( $classes[ array_search('current-menu-item', $classes) ] );
      }

    // Set active state if this is the designs page link
    } elseif ( is_design_archive() && $designs_page == $menu_item->object_id && 'page' === $menu_item->object ) {
      $menu_items[ $key ]->current = true;
      $classes[] = 'current-menu-item';
      $classes[] = 'current_page_item';

    // Set parent state if this is a design page
    } elseif ( is_singular( 'design' ) && $designs_page == $menu_item->object_id ) {
      $classes[] = 'current_page_parent';
    }

    $menu_items[ $key ]->classes = array_unique( $classes );

  }

  return $menu_items;
}
add_filter( 'wp_nav_menu_objects', 'vam_udesign_nav_menu_item_classes', 2 );

/**
 * Fix active class in wp_list_pages for designs page.
 */
function vam_udesign_list_pages( $pages ) {
  if ( is_vam_udesign_page() ) {
    // Remove current_page_parent class from any item.
    $pages = str_replace( 'current_page_parent', '', $pages );
    // Find designs_page_id through udesign options.
    $designs_page = 'page-item-' . vam_udesign_get_page_id( 'designs' );

    if ( is_design_archive() ) {
      // Add current_page_item class to designs page.
      $pages = str_replace( $designs_page, $designs_page . ' current_page_item', $pages );
    } else {
      // Add current_page_parent class to designs page.
      $pages = str_replace( $designs_page, $designs_page . ' current_page_parent', $pages );
    }
  }

  return $pages;
}
add_filter( 'wp_list_pages', 'vam_udesign_list_pages' );

/**
 * Global save post action
 */
function vam_udesign_save_post( $post_id, $post ) {
  // $post_id and $post are required
  if ( empty( $post_id ) || empty( $post ) ) {
    return;
  }

  // Check the post being saved == the $post_id to prevent triggering this call for other save_post events
  if ( empty( $_POST['post_ID'] ) || $_POST['post_ID'] != $post_id ) {
    return;
  }

  // Check for plugin related post type
  $post_type = $post->post_type;
  if ( !in_array( $post_type, array( 'product', 'clipart', 'design' ) ) ) {
    return;
  }

  // Do not save for revisions or autosaves
  if ( defined( 'DOING_AUTOSAVE' ) || is_int( wp_is_post_revision( $post ) ) || is_int( wp_is_post_autosave( $post ) ) ) {
    return;
  }

  // Check user has permission to edit
  if ( ! current_user_can( 'edit_post', $post_id ) ) {
    return;
  }

  // Save search field
  $search_field = $_POST['post_title'];

  $tag_taxonomy = "{$post_type}_tag";
  if ( !empty( $_REQUEST['tax_input'][$tag_taxonomy] ) ) {
     $search_field .= ', ' . $_REQUEST['tax_input'][$tag_taxonomy];
  }

  update_post_meta( $post_id, '_vam_udesign_search_field', strtolower( $search_field ) );
}
add_action( 'save_post', 'vam_udesign_save_post', 10, 2 );
