<?php
/**
 * VAM_UProduct Product Functions
 *
 * Functions for product specific things.
 *
 * @since      1.0.0
 * @package    VAM_UProduct
 * @subpackage VAM_UProduct/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * Get default variation of variable product
 * @since  1.0.0
 * @return int
 */
function vam_udesign_get_product_default_variation( $product_id ) {
  $product = wc_get_product( $product_id );
  if ( empty( $product ) ) {
    return false;
  }
  if ( 'variable' != $product->get_type() ) {
    return false;
  }

  $variations = $product->get_children();
  if ( empty( $variations ) ) {
    return false;
  }

  $default_attributes = VAM_UDesign_WC_Compat::get_default_attributes( $product );
  if ( !empty( $default_attributes ) ) {
    foreach ( $variations as $variation_id ) {
      $variation = wc_get_product( $variation_id );

      $is_default_variation = true;
      foreach ( $variation->get_variation_attributes() as $attribute_name => $attribute ) {
        $name = str_replace( 'attribute_', '', $attribute_name );
        if ( empty( $default_attributes[$name] ) ) {
          continue;
        }

        if ( $default_attributes[$name] != $attribute ) {
          $is_default_variation = false;
          break;
        }
      }

      if ( $is_default_variation ) {
        return $variation_id;
      }
    }
  }

  return $variations[0];
}

/**
 * Get design product properties
 * @since  1.0.0
 * @return int
 */
function vam_udesign_get_design_product_props( $product, $design_id, $selected_attributes = false ) {
  $variations = $product->get_available_variations();
  $attributes = $product->get_variation_attributes();

  if ( empty( $selected_attributes ) ) {
    $selected_attributes = VAM_UDesign_WC_Compat::get_default_attributes( $product );
  }

  $attrs = new stdClass();
  $attrs->attributes = &$attributes;
  $attrs->selected_attributes = &$selected_attributes;

  if ( empty( $design_id ) ) {
    $attrs->variations = &$variations;
    return $attrs;
  }

  $design_excluded_colors = get_post_meta( $design_id, '_design_excluded_colors', true );
  if ( empty( $design_excluded_colors ) ) {
    $attrs->variations = &$variations;
    return $attrs;
  }

  $attrs->variations = array();
  if ( !empty( $variations ) ) {
    foreach( $variations as &$variation ) {
      if ( empty( $variation['attributes']['attribute_pa_color'] ) ) {
        continue;
      }

      $color = $variation['attributes']['attribute_pa_color'];
      if ( in_array( $color, $design_excluded_colors ) ) {
        continue;
      }

      $attrs->variations[] = $variation;
    }
  }

  if ( !empty( $attrs->attributes['pa_color'] ) ) {
    $attribute_colors = array();
    foreach( $attrs->attributes['pa_color'] as $attribute_color ) {
      if ( in_array( $attribute_color, $design_excluded_colors ) ) {
        continue;
      }

      $attribute_colors[] = $attribute_color;
    }

    $attrs->attributes['pa_color'] = &$attribute_colors;
  }

  if ( !empty( $selected_attributes['pa_color'] ) ) {
    if ( in_array( $selected_attributes['pa_color'], $design_excluded_colors ) ) {
      unset( $selected_attributes['pa_color'] );
    }
  }

  return $attrs;
}

/**
 * designer products
 * @since  1.0.0
 * @return int
 */
function vam_udesign_get_products( $args = array() ) {
  // Base query arguments
  $query_args = array(
    'fields'          => 'ids',
    'udesign_query'   => 'product_query',
    'orderby'         => 'title',
    'order'           => 'ASC',
    'post_type'       => 'product',
    'post_status'     => 'publish',
    'posts_per_page'  => -1,
    'tax_query'       => WC()->query->get_tax_query(),
    'meta_query'      => WC()->query->get_meta_query(),
  );

  // Fields
  if ( !empty( $args['fields'] ) ) {
    $query_args['fields'] = $args['fields'];
  }

  // Posts per page
  if ( !empty( $args['posts_per_page'] ) ) {
    $posts_per_page = absint( $args['posts_per_page'] );
    if ( !empty( $posts_per_page ) ) {
      $query_args['posts_per_page'] = $posts_per_page;
    }
  }

  // Page
  if ( !empty( $args['paged'] ) ) {
    $paged = absint( $args['paged'] );
    if ( !empty( $paged ) ) {
      $query_args['paged'] = $paged;
    }
  }

  // Date filter
  if ( !empty( $args['m'] ) ) {
    $query_args['m'] = $args['m'];
  }

  // Sorting
  if ( !empty( $args['orderby'] ) ) {
    $query_args['orderby'] = $args['orderby'];
  }
  if ( !empty( $args['order'] ) ) {
    $query_args['order'] = $args['order'];
  }

  // Tax query
  if ( isset( $args['tax_query'] ) && is_array( $args['tax_query'] ) ) {
    $query_args['tax_query'] = $args['tax_query'];
  }

  // Post type
  if ( !empty( $args['post_type'] ) ) {
    $query_args['post_type'] = $args['post_type'];
  }

  // Product type
  $product_type = 'variable';
  if ( isset( $args['product_type'] ) ) {
    $product_type = $args['product_type'];
  }

  if ( !empty( $product_type ) ) {
    $query_args['tax_query'][] = array(
      'taxonomy' => 'product_type',
      'field'    => 'slug',
      'terms'    => 'variable'
    );
  }

  // From category
  if ( !empty( $args['category_id'] ) ) {
    $query_args['tax_query'][] = array(
      'taxonomy' => 'product_cat',
      'terms' => $args['category_id'],
    );
  }

  // Search query
  if ( !empty( $args['search_query'] ) ) {
    $query_args['meta_query'][] = array(
      'key'     => '_vam_udesign_search_field',
      'value'   => strtolower( sanitize_text_field( $args['search_query'] ) ),
      'compare' => 'LIKE'
    );
  }

  // Search by remote id
  if ( !empty( $args['remote_id'] ) ) {
    $remote_id = $args['remote_id'];
    if ( !is_array( $remote_id ) ) {
      $remote_id = array( $remote_id );
    }

    $query_args['meta_query'][] = array(
      'key'     => '_vam_udesign_remote_id',
      'value'   => $remote_id,
      'compare' => 'in'
    );
  }

  // Designer products
  if ( !empty( $args['designer_products'] ) ) {
    $query_args['meta_query'][] = array(
      'key'     => '_designer',
      'value'   => 'yes',
      'compare' => '='
    );
  }

  // Get products
  $products = new WP_Query( $query_args );
  //wp_reset_postdata();

  return $products;
}


/**
 * Get designer product categories
 * @since  1.0.0
 * @return int
 */
function vam_udesign_get_designer_product_categories( $args ) {
  $query = vam_udesign_get_products( array(
    'designer_products' => true
  ) );

  if ( empty( $query ) ) {
    return array();
  }

  return wp_get_object_terms( $query->posts, 'product_cat', $args );
}
