<?php
/**
 * VAM_UDesign Util Functions
 *
 * Utility functions available both in the frontend and in the admin area
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

function vam_udesign_json_decode( $data, $strip_slashes = false, $assoc = true ) {
  if ( is_array( $data ) ) {
    return $data;
  }

  $result = vam_udesign_try_json_decode( $data, $strip_slashes, $assoc );
  if ( !empty( $result ) ) {
    return $result;
  }

  return vam_udesign_try_json_decode( $data, !$strip_slashes, $assoc );
}

function vam_udesign_try_json_decode( $data, $strip_slashes = false, $assoc = true ) {
  if ( $strip_slashes ) {
    $data = stripslashes( $data );
  }

  return json_decode( $data, $assoc );
}

function vam_udesign_error_response( $message ) {
  return array (
    'success' => false,
    'message' => __( $message, 'vam-udesign' ),
  );
}

function vam_udesign_success_response( $data ) {
  return array (
    'success' => true,
    'data' => $data
  );
}

function vam_udesign_has_props( $arr, $props ) {
  foreach( $props as $prop ) {
    if ( empty( $arr[$prop] ) ) {
      return false;
    }
  }

  return true;
}

function vam_udesign_path_from_url( $url ) {
  $upload_dir = wp_upload_dir();
  $path = str_replace( $upload_dir['baseurl'], $upload_dir['basedir'], $url );

  return realpath( str_replace( '/', DIRECTORY_SEPARATOR, $path ) );
}

function vam_udesign_delete_attachments( $attachment_ids ) {
  if ( empty( $attachment_ids ) ) {
    return;
  }

  foreach( $attachment_ids as $attachment_id ) {
    if ( !empty( $attachment_id ) ) {
      wp_delete_attachment( $attachment_id, true );
    }
  }
}

function vam_udesign_delete_file( $file ) {
  if ( empty( $file ) ) {
    return;
  }

  @unlink( $file );
}

function vam_udesign_delete_url( $url ) {
  if ( empty( $url ) ) {
    return;
  }

  vam_udesign_delete_file( vam_udesign_path_from_url( $url ) );
}

function vam_udesign_post_dates( $post_type ) {
  global $wpdb, $wp_locale;

  $query = "SELECT DISTINCT YEAR(post_date) AS yyear, MONTH(post_date) AS mmonth
    FROM $wpdb->posts WHERE post_type = '$post_type' ORDER BY post_date DESC";
  $results = $wpdb->get_results( $query );

  $month_count = count( $results );
  if ( !$month_count || ( 1 == $month_count && 0 == $results[0]->mmonth ) ) {
    return array();
  }

  $months = array();
  foreach ( $results as &$result ) {
    if ( $result->yyear == 0 ) {
      continue;
    }

    $result->mmonth = zeroise( $result->mmonth, 2 );
    $key = esc_attr( $result->yyear . $result->mmonth );
    $value = sprintf( '%s %d', $wp_locale->get_month( $result->mmonth ), $result->yyear );

    $months[$key] = esc_html( $value );
  }

  return $months;
}

function vam_udesign_remote_request( $url, $args = array(), $json = false ) {
  if ( empty( $args ) ) {
    $args = array();
  }
  if ( empty( $args['method'] ) ) {
    $args['method'] = 'GET';
  }

  $response = wp_remote_request( $url, $args );
  if ( empty( $response ) ) {
    return false;
  }

  $status = wp_remote_retrieve_response_code( $response );
  if ( empty( $status ) || absint( $status ) != 200 ) {
    return false;
  }

  $data = wp_remote_retrieve_body( $response );
  if ( empty( $data ) || is_wp_error( $data ) ) {
    return false;
  }

  return $json ? vam_udesign_json_decode( $data ) : $data;
}

function vam_udesign_upload_image( $file, $is_blob = false, $name = '' ) {
  $uploaded_file = ( $is_blob ) ? wp_upload_bits( $name, null, $file ) :
    wp_handle_upload( $file, array( 'test_form' => false ) );

  if ( empty( $uploaded_file ) || !empty( $uploaded_file['error'] ) ) {
    return false;
  }

  return $uploaded_file;
}

function vam_udesign_upload_remote_image( $url, $name = '' ) {
  $image = vam_udesign_remote_request( $url );
  if ( empty( $image ) ) {
    return false;
  }

  if ( empty( $name ) ) {
    $name = basename( $url );
  }

  return vam_udesign_upload_image( $image, true, $name );
}

function vam_udesign_create_attachment( $file, $post_id = 0 ) {
  $upload_dir = wp_upload_dir();
  $filepath = $file['file'];

  $title = get_the_title( $post_id );
  if ( empty( $title ) ) {
    $title = preg_replace( '/\.[^.]+$/', '', basename( $filepath ) );
  }

  $attachment = array(
    'guid'           => $upload_dir['url'] . '/' . basename( $filepath ),
    'post_mime_type' => $file['type'],
    'post_title'     => $title,
    'post_content'   => '',
    'post_status'    => 'inherit'
  );

  $attach_id = wp_insert_attachment( $attachment, $filepath, $post_id );
  if ( empty( $attach_id ) ) {
    return false;
  }

  require_once( ABSPATH . 'wp-admin/includes/image.php' );
  $attach_data = wp_generate_attachment_metadata( $attach_id, $filepath );
  wp_update_attachment_metadata( $attach_id, $attach_data );

  return $attach_id;
}

function vam_udesign_create_term( $name, $taxonomy, $search_field = 'slug' ) {
  $name = trim( $name );

  $term = get_term_by( $search_field, $name, $taxonomy, 'ARRAY_A' );
  if ( !empty( $term ) && !empty( $term['term_id'] ) ) {
    return $term['term_id'];
  }

  $term = wp_insert_term( $name, $taxonomy );
  if ( empty ( $term ) || empty( $term['term_id'] ) || is_wp_error( $term ) ) {
    return false;
  }

  return $term['term_id'];
}

function vam_udesign_get_image_size( $image_size ) {
  $size = array(
    'width' => '360',
    'height' => '460',
    'crop' => 1
  );

  if ( !is_string( $image_size ) ) {
    return $size;
  }

  $image_size = trim( $image_size );
  if ( $image_size == '' ) {
    return $size;
  }

  $image_sizes = array(
    'udesign_design_catalog' => array(
      'width' => '200',
      'height' => '260',
      'crop' => 1
    ),

    'udesign_design_single' => array(
      'width' => '360',
      'height' => '460',
      'crop' => 1
    ),

    'udesign_design_thumbnail' => array(
      'width' => '200',
      'height' => '260',
      'crop' => 1
    )
  );

  if ( !array_key_exists( $image_size, $image_sizes ) ) {
    return $size;
  }
  $default_size = $image_sizes[$image_size];

  $size = get_option( $image_size . '_image_size', $default_size );
  if ( !isset( $size['width'] ) ) {
    $size['width'] = $default_size['width'];
  }
  if ( !isset( $size['height'] ) ) {
    $size['height'] = $default_size['height'];
  }
  if ( !isset( $size['crop'] ) ) {
    $size['crop'] = $default_size['crop'];
  }
  if ( empty( $size['width'] ) && empty( $size['height'] ) ) {
    $size = $default_size;
  }

  return apply_filters( 'vam_udesign_get_image_size_' . $image_size, $size );
}

function vam_udesign_get_image_size_array( $image_size ) {
  $size = vam_udesign_get_image_size( $image_size );
  return array(
    $size['width'],
    $size['height'],
    $size['crop']
  );
}

function vam_udesign_generate_thumbnail( $path, $name, $width, $height, $to = false ) {
  if ( empty( $path ) ) {
    return false;
  }
  if ( empty( $width ) && empty( $height ) ) {
    return false;
  }

  // load source image
  $src = imagecreatefrompng( $path );
  if ( empty( $src ) ) {
    return false;
  }

  // check destination image width and height
  $src_width = imagesx( $src );
  $src_height = imagesy( $src );

  if ( empty( $width ) ) {
    $width = $height * ( $src_height / $src_width );
  }
  if ( empty( $height ) ) {
    $height = $width * ( $src_width / $src_height );
  }

  // create destination image
  $dest = imagecreatetruecolor( $width, $height );
  if ( empty( $dest ) ) {
    imagedestroy( $src );
    return false;
  }
  if ( empty( imagealphablending( $dest, true ) ) ) {
    imagedestroy( $src );
    imagedestroy( $dest );
    return false;
  }
  if ( empty( imagesavealpha( $dest, true ) ) ) {
    imagedestroy( $src );
    imagedestroy( $dest );
    return false;
  }

  // give destination image a transparent background
  $background_color = imagecolorallocatealpha( $dest, 255, 255, 255, 127 );
  if ( empty( $background_color ) ) {
    imagedestroy( $src );
    imagedestroy( $dest );
    return false;
  }
  if ( empty( imagefill( $dest, 0, 0, $background_color ) ) ) {
    imagedestroy( $src );
    imagedestroy( $dest );
    return false;
  }

  // scale source image to fit into the destination image
  $scale = $height / $src_height;
  if ( $src_width / $src_height > $width / $height ) {
      $scale = $width / $src_width;
  }
  $scaled_width = $src_width * $scale;
  $scaled_height = $src_height * $scale;

  // copy source image onto the destination image
  $x = ( $width - $scaled_width ) / 2;
  $y = ( $height - $scaled_height ) / 2;

  if ( empty( imagecopyresampled( $dest, $src, $x, $y, 0, 0, $scaled_width, $scaled_height, $src_width, $src_height ) ) ) {
    imagedestroy( $src );
    imagedestroy( $dest );
    return false;
  }
  imagedestroy( $src );

  // convert destination image to png
  if ( !empty( $to ) ) {
    $result = imagepng( $dest, $to );
    imagedestroy( $dest );
    return $result;
  }

  ob_start();
  imagepng( $dest, null );
  $dest_data = ob_get_contents();
  ob_end_clean();

  // upload destination image
  $uploaded_file = wp_upload_bits( $name, null, $dest_data );
  if ( empty( $uploaded_file ) || !empty( $uploaded_file['error'] ) ) {
    imagedestroy( $dest );
    return false;
  }

  imagedestroy( $dest );
  return $uploaded_file;
}
