<?php
/**
 * Handle frontend scripts
 *
 * @class       VAM_UDesign_Frontend_Scripts
 * @version     1.0.0
 * @package     VAM_UDesign
 * @subpackage  VAM_UDesign/public
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_Frontend_Scripts Class.
 */
class VAM_UDesign_Frontend_Scripts {

  /**
   * Contains an array of script handles registered by plugin.
   * @var array
   */
  private static $scripts = array();

  /**
   * Contains an array of script handles registered by plugin.
   * @var array
   */
  private static $styles = array();

  /**
   * Contains an array of script handles localized by plugin.
   * @var array
   */
  private static $wp_localize_scripts = array();

  /**
   * Hook in methods.
   */
  public static function init() {
    add_action( 'wp_enqueue_scripts', array( __CLASS__, 'load_scripts' ) );
    add_action( 'wp_print_scripts', array( __CLASS__, 'localize_printed_scripts' ), 5 );
    add_action( 'wp_print_footer_scripts', array( __CLASS__, 'localize_printed_scripts' ), 5 );
  }

  /**
   * Get styles for the frontend.
   * @access private
   * @return array
   */
  public static function get_styles() {
    return apply_filters( 'vam_udesign_enqueue_styles', array(
      'vam-udesign' => array(
        'src'     => str_replace( array( 'http:', 'https:' ), '', self::get_asset_url( 'assets/css/vam-udesign-public.css' ) ),
        'deps'    => '',
        'version' => VAM_UDESIGN_VERSION,
        'media'   => 'all',
        'has_rtl' => false,
      ),
    ) );
  }

  /**
   * Get woocommerce styles for the frontend.
   * @access public
   * @return array
   */
  public static function get_woocommerce_styles() {
    return apply_filters( 'vam_udesign_woocommerce_enqueue_styles', array(
      'woocommerce-layout' => array(
        'src'     => str_replace( array( 'http:', 'https:' ), '', WC()->plugin_url() ) . '/assets/css/woocommerce-layout.css',
        'deps'    => '',
        'version' => WC_VERSION,
        'media'   => 'all',
        'has_rtl' => false,
      ),
      'woocommerce-smallscreen' => array(
        'src'     => str_replace( array( 'http:', 'https:' ), '', WC()->plugin_url() ) . '/assets/css/woocommerce-smallscreen.css',
        'deps'    => 'woocommerce-layout',
        'version' => WC_VERSION,
        'media'   => 'only screen and (max-width: ' . apply_filters( 'woocommerce_style_smallscreen_breakpoint', $breakpoint = '768px' ) . ')',
        'has_rtl' => false,
      ),
      'woocommerce-general' => array(
        'src'     => str_replace( array( 'http:', 'https:' ), '', WC()->plugin_url() ) . '/assets/css/woocommerce.css',
        'deps'    => '',
        'version' => WC_VERSION,
        'media'   => 'all',
        'has_rtl' => false,
      ),
    ) );
  }

  /**
   * Return protocol relative asset URL.
   * @param string $path
   */
  private static function get_asset_url( $path ) {
    return str_replace( array( 'http:', 'https:' ), '', VAM_UDesign()->plugin_url( $path ) );
  }

  /**
   * Register a script for use.
   *
   * @uses   wp_register_script()
   * @access private
   * @param  string   $handle
   * @param  string   $path
   * @param  string[] $deps
   * @param  string   $version
   * @param  boolean  $in_footer
   */
  private static function register_script( $handle, $path, $deps = array( 'jquery' ), $version = WC_VERSION, $in_footer = true ) {
    self::$scripts[] = $handle;
    wp_register_script( $handle, $path, $deps, $version, $in_footer );
  }

  /**
   * Register and enqueue a script for use.
   *
   * @uses   wp_enqueue_script()
   * @access private
   * @param  string   $handle
   * @param  string   $path
   * @param  string[] $deps
   * @param  string   $version
   * @param  boolean  $in_footer
   */
  private static function enqueue_script( $handle, $path = '', $deps = array( 'jquery' ), $version = WC_VERSION, $in_footer = true ) {
    if ( ! in_array( $handle, self::$scripts ) && $path ) {
      self::register_script( $handle, $path, $deps, $version, $in_footer );
    }
    wp_enqueue_script( $handle );
  }

  /**
   * Register a style for use.
   *
   * @uses   wp_register_style()
   * @access private
   * @param  string   $handle
   * @param  string   $path
   * @param  string[] $deps
   * @param  string   $version
   * @param  string   $media
   * @param  boolean  $has_rtl
   */
  private static function register_style( $handle, $path, $deps = array(), $version = WC_VERSION, $media = 'all', $has_rtl = false ) {
    self::$styles[] = $handle;
    wp_register_style( $handle, $path, $deps, $version, $media );

    if ( $has_rtl ) {
      wp_style_add_data( $handle, 'rtl', 'replace' );
    }
  }

  /**
   * Register and enqueue a styles for use.
   *
   * @uses   wp_enqueue_style()
   * @access private
   * @param  string   $handle
   * @param  string   $path
   * @param  string[] $deps
   * @param  string   $version
   * @param  string   $media
   * @param  boolean  $has_rtl
   */
  private static function enqueue_style( $handle, $path = '', $deps = array(), $version = WC_VERSION, $media = 'all', $has_rtl = false ) {
    if ( ! in_array( $handle, self::$styles ) && $path ) {
      self::register_style( $handle, $path, $deps, $version, $media, $has_rtl );
    }
    wp_enqueue_style( $handle );
  }

  /**
   * Register all scripts.
   */
  private static function register_scripts() {
    $suffix           = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
    $register_scripts = array(
      'jquery-blockui' => array(
        'src'     => self::get_asset_url( 'assets/js/jquery-blockui/jquery.blockUI' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => '2.70.0',
      ),
      'canvas-to-blob' => array(
        'src'     => self::get_asset_url( 'assets/js/canvas-to-blob/canvas-to-blob' . $suffix . '.js' ),
        'deps'    => array(),
        'version' => '3.14.0',
      ),
      'vam-udesign' => array(
        'src' => self::get_asset_url( 'assets/js/vam-udesign' . $suffix . '.js' ),
        'deps' => array( 'jquery', 'jquery-blockui', 'canvas-to-blob' ),
        'version' => VAM_UDESIGN_VERSION,
      ),
      'flexslider' => array(
        'src'     => self::get_asset_url( 'assets/js/flexslider/jquery.flexslider' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => '2.6.1',
      ),
      'photoswipe' => array(
        'src'     => self::get_asset_url( 'assets/js/photoswipe/photoswipe' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => '4.1.2',
      ),
      'photoswipe-ui-default'  => array(
        'src'     => self::get_asset_url( 'assets/js/photoswipe/photoswipe-ui-default' . $suffix . '.js' ),
        'deps'    => array( 'photoswipe' ),
        'version' => '4.1.2',
      ),
      'zoom' => array(
        'src'     => self::get_asset_url( 'assets/js/zoom/jquery.zoom' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => '1.7.20',
      ),
      'isotope-layout' => array(
        'src'     => self::get_asset_url( 'assets/js/isotope-layout/isotope.pkgd' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => '3.0.3',
      ),
      'owl-carousel' => array(
        'src'     => self::get_asset_url( 'assets/js/owl-carousel/owl.carousel' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => '2.2.0',
      ),
      'design-add-to-cart' => array(
        'src'     => self::get_asset_url( 'assets/js/design-add-to-cart' . $suffix . '.js' ),
        'deps'    => array( 'jquery', 'wp-util' ),
        'version' => VAM_UDESIGN_VERSION,
      ),
      'single-design' => array(
        'src'     => self::get_asset_url( 'assets/js/single-design' . $suffix . '.js' ),
        'deps'    => array( 'jquery', 'jquery-blockui' ),
        'version' => VAM_UDESIGN_VERSION,
      ),
      'design-gallery' => array(
        'src'     => self::get_asset_url( 'assets/js/design-gallery' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => VAM_UDESIGN_VERSION,
      ),
      'single-customize' => array(
        'src'     => self::get_asset_url( 'assets/js/single-customize' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => VAM_UDESIGN_VERSION,
      ),
      'single-swatches' => array(
        'src'     => self::get_asset_url( 'assets/js/single-swatches' . $suffix . '.js' ),
        'deps'    => array( 'jquery' ),
        'version' => VAM_UDESIGN_VERSION,
      ),
      'add-to-cart-design' => array(
        'src'     => self::get_asset_url( 'assets/js/add-to-cart-design' . $suffix . '.js' ),
        'deps'    => array( 'jquery', 'vam-udesign' ),
        'version' => VAM_UDESIGN_VERSION,
      ),
    );

    foreach ( $register_scripts as $name => $props ) {
      self::register_script( $name, $props['src'], $props['deps'], $props['version'] );
    }
  }

  /**
   * Register all styles.
   */
  private static function register_styles() {
    $register_styles = array(
      'owl-carousel' => array(
        'src'     => self::get_asset_url( 'assets/css/owl.carousel.css' ),
        'deps'    => array(),
        'version' => '2.2.0',
        'has_rtl' => false,
      ),
      'photoswipe' => array(
        'src'     => self::get_asset_url( 'assets/css/photoswipe/photoswipe.css' ),
        'deps'    => array(),
        'version' => '4.1.2',
        'has_rtl' => false,
      ),
      'photoswipe-default-skin' => array(
        'src'     => self::get_asset_url( 'assets/css/photoswipe/default-skin/default-skin.css' ),
        'deps'    => array( 'photoswipe' ),
        'version' => '4.1.2',
        'has_rtl' => false,
      ),
    );

    foreach ( $register_styles as $name => $props ) {
      self::register_style( $name, $props['src'], $props['deps'], $props['version'], 'all', $props['has_rtl'] );
    }
  }

  /**
   * Register/queue frontend scripts.
   */
  public static function load_scripts() {
    global $post;

    if ( ! did_action( 'before_woocommerce_init' ) ) {
      return;
    }

    self::register_scripts();
    self::register_styles();

    $suffix               = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
    $lightbox_en          = 'yes' === get_option( 'woocommerce_enable_lightbox' );
    $ajax_cart_en         = 'yes' === get_option( 'woocommerce_enable_ajax_add_to_cart' );
    $products_list_layout = get_option( 'udesign_products_list_layout', 'carousel' );
    $assets_path          = str_replace( array( 'http:', 'https:' ), '', VAM_UDesign()->plugin_url() ) . '/assets/';
    $frontend_script_path = $assets_path . 'js/';
    $wc_assets_path       = str_replace( array( 'http:', 'https:' ), '', WC()->plugin_url() ) . '/assets/';
    $wc_frontend_script_path = $wc_assets_path . 'js/frontend/';

    // Enqueue frontend scripts conditionally
    if ( is_design() || is_product() ) {
      self::enqueue_script( 'single-swatches' );
      self::enqueue_script( 'single-customize' );
    }
    if ( $products_list_layout == 'masonry' && is_design() ) {
      self::enqueue_script( 'isotope-layout' );
    }
    if ( $products_list_layout == 'carousel' && is_design() ) {
      self::enqueue_script( 'owl-carousel' );
      self::enqueue_style( 'owl-carousel' );
    }
    if ( is_design() ) {
      self::enqueue_script( 'design-add-to-cart' );
      self::enqueue_script( 'single-design' );
      self::enqueue_script( 'add-to-cart-design' );
      self::enqueue_script( 'zoom' );
      self::enqueue_script( 'flexslider' );
      self::enqueue_script( 'photoswipe-ui-default' );
      self::enqueue_script( 'design-gallery' );
      self::enqueue_style( 'photoswipe' );
      self::enqueue_style( 'photoswipe-default-skin' );
      add_action( 'wp_footer', 'vam_udesign_photoswipe' );
    }
    if ( is_designer_page() ) {
      self::enqueue_script( 'single-swatches' );
      self::enqueue_script( 'single-design' );
      self::enqueue_script( 'add-to-cart-design' );
      self::enqueue_script( 'design-add-to-cart' );
    }

   // Styles
    if ( $enqueue_styles = self::get_styles() ) {
      foreach ( $enqueue_styles as $handle => $args ) {
        if ( ! isset( $args['has_rtl'] ) ) {
          $args['has_rtl'] = false;
        }

        self::enqueue_style( $handle, $args['src'], $args['deps'], $args['version'], $args['media'], $args['has_rtl'] );
      }
    }

    if ( is_vam_udesign_page() ) {
      if ( $enqueue_woocommerce_styles = self::get_woocommerce_styles() ) {
        foreach ( $enqueue_woocommerce_styles as $handle => $args ) {
          wp_enqueue_style( $handle, $args['src'], $args['deps'], $args['version'], $args['media'] );
        }
      }
    }
  }

  /**
   * Localize a script once.
   * @access private
   * @since  1.0.0
   * @param  string $handle
   */
  private static function localize_script( $handle ) {
    if ( ! in_array( $handle, self::$wp_localize_scripts ) && wp_script_is( $handle ) && ( $data = self::get_script_data( $handle ) ) ) {
      $name                        = str_replace( '-', '_', $handle ) . '_params';
      self::$wp_localize_scripts[] = $handle;
      wp_localize_script( $handle, $name, apply_filters( $name, $data ) );
    }
  }

  /**
   * Return data for script handles.
   * @access private
   * @param  string $handle
   * @return array|bool
   */
  private static function get_script_data( $handle ) {
    global $wp, $post;

    switch ( $handle ) {
      case 'single-swatches':
        return array(
          'ajax_url' => WC()->ajax_url()
        );

      case 'design-add-to-cart' :
        vam_udesign_get_template( 'single-design/add-to-cart/variation.php' );

        return array(
          'ajax_url'                         => WC()->ajax_url(),
          'wc_ajax_url'                      => WC_AJAX::get_endpoint( "%%endpoint%%" ),
          'i18n_no_matching_variations_text' => esc_attr__( 'Sorry, no products matched your selection. Please choose a different combination.', 'vam-udesign' ),
          'i18n_make_a_selection_text'       => esc_attr__( 'Please select some product options before adding this product to your cart.', 'vam-udesign' ),
          'i18n_unavailable_text'            => esc_attr__( 'Sorry, this product is unavailable. Please choose a different combination.', 'vam-udesign' ),
          'get_design_preview_nonce'         => wp_create_nonce( 'get-design-preview' ),
          'get_design_price_nonce'           => wp_create_nonce( 'designer' ),
          'is_designer_page'                 => is_designer_page()
        );
      break;
      case 'single-design' :
        return array(
          'design_id'                   => ( isset( $post->ID ) && isset( $post->post_type ) && $post->post_type == 'design' ) ? $post->ID : '',
          'ajax_url'                    => WC()->ajax_url(),
          'load_designer_product_nonce' => wp_create_nonce( 'load-designer-product' ),
          'products_list_layout'        => get_option( 'udesign_products_list_layout', 'carousel' ),
          'is_designer_page'            => is_designer_page()
        );
      break;
      case 'design-gallery' :
        return array(
          'flexslider'                => apply_filters( 'vam_udesign_design_gallery_options', array(
            'rtl'            => is_rtl(),
            'animation'      => 'slide',
            'smoothHeight'   => false,
            'directionNav'   => false,
            'controlNav'     => 'thumbnails',
            'slideshow'      => false,
            'animationSpeed' => 500,
            'animationLoop'  => false, // Breaks photoswipe pagination if true.
          ) ),
          'zoom_enabled'       => true,
          'photoswipe_enabled' => true,
          'flexslider_enabled' => true,
        );
      break;
      case 'single-customize' :
        $designer_page_id  = get_option( 'udesign_designer_page_id' );
        $designer_page_url = $designer_page_id ? get_permalink( $designer_page_id ) : false;

        return array(
          'post_id'                     => isset( $post->ID ) ? $post->ID : '',
          'designer_page_url'           => $designer_page_url,
          'i18n_make_a_selection_text'  => esc_js( __( 'Please select color and size before customizing this product.', 'vam-udesign' ) ),
          'i18n_unavailable_text'       => esc_js( __( 'Sorry, this product is unavailable. Please choose a different combination.', 'vam-udesign' ) ),
        );
      break;
      case 'vam-udesign' :
        return array(
          'ajax_url' => WC()->ajax_url(),
        );
      break;
    }

    return false;
  }

  /**
   * Localize scripts only when enqueued.
   */
  public static function localize_printed_scripts() {
    foreach ( self::$scripts as $handle ) {
      self::localize_script( $handle );
    }
  }
}

VAM_UDesign_Frontend_Scripts::init();
