<?php
/**
 * Template Loader
 *
 * @class     VAM_UDesign_Template_Loader
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

class VAM_UDesign_Template_Loader {

  /**
   * Hook in methods.
   */
  public static function init() {
    add_filter( 'template_include', array( __CLASS__, 'template_loader' ) );
    add_filter( 'comments_template', array( __CLASS__, 'comments_template_loader' ) );
  }
  /**
   * Load a template.
   *
   * Handles template usage so that we can use our own templates instead of the themes.
   *
   * Templates are in the 'templates' folder.
   * Looks for theme overrides in /theme/udesign/ by default.
   *
   * For beginners, it also looks for a udesign.php template first. If the user adds.
   * this to the theme (containing a udesign() inside) this will be used for all.
   * udesign templates.
   *
   * @param mixed $template
   * @return string
   */
  public static function template_loader( $template ) {
    if ( is_embed() ) {
      return $template;
    }
    if ( $default_file = self::get_template_loader_default_file() ) {
      /**
       * Filter hook to choose which files to find before WooCommerce does it's own logic.
       *
       * @since 1.0.0
       * @var array
       */
      $search_files = self::get_template_loader_files( $default_file );
      $template     = locate_template( $search_files );
      if ( ! $template || WC_TEMPLATE_DEBUG_MODE ) {
        $template = VAM_UDesign()->plugin_path() . '/public/templates/' . $default_file;
      }
    }

    return $template;
  }
  /**
   * Get the default filename for a template.
   *
   * @since  1.0.0
   * @return string
   */
  private static function get_template_loader_default_file() {
    if ( is_singular( 'design' ) ) {
      $default_file = 'single-design.php';
    } elseif ( is_tax( get_object_taxonomies( 'design' ) ) ) {
      $term = get_queried_object();
      if ( is_tax( 'design_cat' ) || is_tax( 'design_tag' ) ) {
        $default_file = 'taxonomy-' . $term->taxonomy . '.php';
      } else {
        $default_file = 'archive-design.php';
      }
    } elseif ( is_post_type_archive( 'design' ) || is_page( get_option( 'udesign_designs_page_id' ) ) ) {
      $default_file = 'archive-design.php';
    } else {
      $default_file = '';
    }

    return $default_file;
  }
  /**
   * Get an array of filenames to search for a given template.
   *
   * @since  1.0.0
   * @param  string $file The default file name.
   * @return string[]
   */
  private static function get_template_loader_files( $default_file ) {
    $search_files   = array();
    if ( is_tax( get_object_taxonomies( 'design' ) ) ) {
      $term   = get_queried_object();
      $search_files[] = 'taxonomy-' . $term->taxonomy . '-' . $term->slug . '.php';
      $search_files[] = VAM_UDesign()->template_path() . 'taxonomy-' . $term->taxonomy . '-' . $term->slug . '.php';
      $search_files[] = 'taxonomy-' . $term->taxonomy . '.php';
      $search_files[] = VAM_UDesign()->template_path() . 'taxonomy-' . $term->taxonomy . '.php';
    }

    $search_files[] = $default_file;
    $search_files[] = VAM_UDesign()->template_path() . $default_file;

    return array_unique( $search_files );
  }
  /**
   * Load comments template.
   *
   * @since  1.0.0
   * @param mixed $template
   * @return string
   */
  public static function comments_template_loader( $template ) {
    if ( get_post_type() !== 'design' ) {
      return $template;
    }

    $check_dirs = array(
      trailingslashit( get_stylesheet_directory() ) . VAM_UDesign()->template_path(),
      trailingslashit( get_template_directory() ) . VAM_UDesign()->template_path(),
      trailingslashit( get_stylesheet_directory() ),
      trailingslashit( get_template_directory() ),
      trailingslashit( VAM_UDesign()->plugin_path() ) . 'templates/',
    );

    foreach ( $check_dirs as $dir ) {
      if ( file_exists( trailingslashit( $dir ) . 'single-design-reviews.php' ) ) {
        return trailingslashit( $dir ) . 'single-design-reviews.php';
      }
    }
  }
}

VAM_UDesign_Template_Loader::init();
