<?php
/**
 * VAM_UDesign Template
 *
 * Functions for the templating system.
 *
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 * @since      1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * When the_post is called, put design data into a global.
 *
 * @param mixed $post
 * @return VAM_UDesign_Design
 */
function vam_udesign_setup_design_data( $post ) {
  unset( $GLOBALS['design'] );

  if ( is_int( $post ) )
    $post = get_post( $post );

  if ( empty( $post->post_type ) || ! in_array( $post->post_type, array( 'design' ) ) )
    return;

  $GLOBALS['design'] = vam_udesign_get_design( $post );

  return $GLOBALS['design'];
}
add_action( 'the_post', 'vam_udesign_setup_design_data' );

if ( ! function_exists( 'vam_udesign_reset_loop' ) ) {

  /**
   * Reset the loop's index and columns when we're done outputting a design loop.
   */
  function vam_udesign_reset_loop() {
    $GLOBALS['udesign_loop'] = array(
      'loop'    => '',
      'columns' => '',
      'name'    => '',
    );
  }
}
add_filter( 'loop_end', 'vam_udesign_reset_loop' );

/**
 * Get template part (for templates like the shop-loop).
 *
 * @access public
 * @param mixed $slug
 * @param string $name (default: '')
 */
function vam_udesign_get_template_part( $slug, $name = '' ) {
  $template = '';

  // Look in  yourtheme/udesign/slug-name.php
  if ( $name ) {
    $template = locate_template( array( VAM_UDesign()->template_path() . "{$slug}-{$name}.php" ) );
  }

  // Get default slug-name.php
  if ( ! $template && $name && file_exists( VAM_UDesign()->plugin_path() . "/public/templates/{$slug}-{$name}.php" ) ) {
    $template = VAM_UDesign()->plugin_path() . "/public/templates/{$slug}-{$name}.php";
  }

  // If template file doesn't exist, look in yourtheme/slug.php and yourtheme/udesign/slug.php
  if ( ! $template && ! WC_TEMPLATE_DEBUG_MODE ) {
    $template = locate_template( array( "{$slug}.php", WC()->template_path() . "{$slug}.php" ) );
  }

  if ( $template ) {
    load_template( $template, false );
  }
}

/**
 * Get other templates (e.g. design attributes) passing attributes and including the file.
 *
 * @access public
 * @param string $template_name
 * @param array $args (default: array())
 * @param string $template_path (default: '')
 * @param string $default_path (default: '')
 */
function vam_udesign_get_template( $template_name, $args = array(), $template_path = '', $default_path = '' ) {
  if ( ! empty( $args ) && is_array( $args ) ) {
    extract( $args );
  }

  $located = vam_udesign_locate_template( $template_name, $template_path, $default_path );

  if ( ! file_exists( $located ) ) {
    _doing_it_wrong( __FUNCTION__, sprintf( '<code>%s</code> does not exist.', $located ), '1.0.0' );
    return;
  }

  include( $located );
}

/**
 * Like vam_udesign_get_template, but returns the HTML instead of outputting.
 *
 * @see vam_udesign_get_template
 * @param string $template_name
 * @since 1.0.0
 */
function vam_udesign_get_template_html( $template_name, $args = array(), $template_path = '', $default_path = '' ) {
  ob_start();
  vam_udesign_get_template( $template_name, $args, $template_path, $default_path );
  return ob_get_clean();
}

/**
 * Locate a template and return the path for inclusion.
 *
 * This is the load order:
 *
 *    yourtheme   / $template_path  / $template_name
 *    yourtheme   / $template_name
 *    $default_path / $template_name
 *
 * @access public
 * @param string $template_name
 * @param string $template_path (default: '')
 * @param string $default_path (default: '')
 * @return string
 */
function vam_udesign_locate_template( $template_name, $template_path = '', $default_path = '' ) {
  if ( ! $template_path ) {
    $template_path = VAM_UDesign()->template_path();
  }

  if ( ! $default_path ) {
    $default_path = VAM_UDesign()->plugin_path() . '/public/templates/';
  }

  // Look within passed path within the theme - this is priority.
  $template = locate_template(
    array(
      trailingslashit( $template_path ) . $template_name,
      $template_name
    )
  );

  // Get default template/
  if ( ! $template ) {
    $template = $default_path . $template_name;
  }

  // Return what we found.
  return $template;
}

/**
 * Output an html element start tag.
 *
 * @param array $args
 * @since 1.0.0
 */
function vam_udesign_html_start_el( $args = array(), $echo = false ) {
  $args = wp_parse_args( $args, array(
    'element' => 'div',
    'id'      => '',
    'class'   => '',
  ));

  extract( $args );

  $html = sprintf ( '<%s id="%s" class="%s">', strip_tags( $element ), strip_tags( $id ), strip_tags( $class ) );

  if ( $echo )
    echo $html;
  else
    return $html;
}

/**
 * Output an html element end tag.
 *
 * @param array $args
 * @since 1.0.0
 */
function vam_udesign_html_end_el( $args = array(), $echo = false ) {
  $args = wp_parse_args( $args, array(
    'element' => 'div',
  ));

  extract( $args );

  $html = sprintf ( '</%s>', strip_tags( $element ) );

  if ( $echo )
    echo $html;
  else
    return $html;
}

/**
 * Add body classes for VAM UDesign pages.
 *
 * @param  array $classes
 * @return array
 */
function vam_udesign_body_class( $classes ) {
  $classes = (array) $classes;

  if ( is_singular( 'design' ) || is_post_type_archive( 'design' ) || is_tax( get_object_taxonomies( 'design' ) ) ) {
    $classes[] = 'woocommerce';
    $classes[] = 'woocommerce-page';
    $classes[] = 'udesign';
    $classes[] = 'udesign-page';
  }

  return array_unique( $classes );
}

/**
 * Display the classes for the design cat div.
 *
 * @since 1.0.0
 * @param string|array $class One or more classes to add to the class list.
 * @param object $category object Optional.
 */
function vam_udesign_design_cat_class( $class = '', $category = null ) {
  echo 'class="' . esc_attr( join( ' ', vam_udesign_get_design_cat_class( $class, $category ) ) ) . '"';
}

/**
 * Get classname for loops based on $udesign_loop global.
 *
 * @since 1.0.0
 * @return string
 */
function vam_udesign_get_loop_class() {
  global $udesign_loop;

  $udesign_loop['loop']    = ! empty( $udesign_loop['loop'] ) ? $udesign_loop['loop'] + 1   : 1;
  $udesign_loop['columns'] = ! empty( $udesign_loop['columns'] ) ? $udesign_loop['columns'] : apply_filters( 'loop_shop_columns', 4 );

  if ( 0 === ( $udesign_loop['loop'] - 1 ) % $udesign_loop['columns'] || 1 === $udesign_loop['columns'] ) {
    return 'first';
  } elseif ( 0 === $udesign_loop['loop'] % $udesign_loop['columns'] ) {
    return 'last';
  } else {
    return '';
  }
}

/**
 * Get the classes for the design cat div.
 *
 * @since 1.0.0
 * @param string|array $class One or more classes to add to the class list.
 * @param object $category object Optional.
 */
function vam_udesign_get_design_cat_class( $class = '', $category = null ) {
  $classes   = is_array( $class ) ? $class : array_map( 'trim', explode( ' ', $class ) );
  $classes[] = 'product-category';
  $classes[] = 'product';
  $classes[] = 'design-category';
  $classes[] = 'design';
  $classes[] = vam_udesign_get_loop_class();
  $classes   = apply_filters( 'design_cat_class', $classes, $class, $category );

  return array_unique( array_filter( $classes ) );
}

/**
 * Adds extra post classes for design.
 *
 * @param array $classes
 * @param string|array $class
 * @param int $post_id
 * @return array
 */
function vam_udesign_design_post_class( $classes, $class = '', $post_id = '' ) {
  if ( ! $post_id || 'design' !== get_post_type( $post_id ) ) {
    return $classes;
  }

  $design = vam_udesign_get_design( $post_id );

  if ( $design ) {
    $classes[] ='product';
    $classes[] = vam_udesign_get_loop_class();
  }

  if ( false !== ( $key = array_search( 'hentry', $classes ) ) ) {
    unset( $classes[ $key ] );
  }

  return $classes;
}

/**
 *
 * @param array $args
 * @since 1.0.0
 */
function vam_udesign_wc_swatches_variation_attribute_options_args( $args ) {
  return (array) wp_parse_args( array(
    'class'    => 'wc-default-select',
  ), $args );
}

/**
 * Output a list of variation attributes for use in the cart forms.
 *
 * @param array $html
 * @param array $args
 * @since 1.0.0
 */
function vam_udesign_wc_swatches_variation_attribute_options( $html, $args = array() ) {
  $attribute        = $args['attribute'];
  $attribute_name   = str_replace( 'pa_', '', $attribute );
  $taxonomies       = wp_list_pluck( wc_get_attribute_taxonomies(), 'attribute_type', 'attribute_name' );
  $taxonomy_type    = isset( $taxonomies[ $attribute_name ] ) ? $taxonomies[ $attribute_name ] : 'select';

  if ( $taxonomy_type != 'custom' ) {
    echo $html;

    return;
  }

  $options          = $args['options'];
  $product          = $args['product'];
  $id               = $args['id'] ? $args['id'] : sanitize_title( $attribute );
  $swatch_html = '<div id="picker_' . esc_attr( $id ) . '" class="select swatch-control">';
  $swatch_html .= str_replace('wc-default-select', '', $html);

  if ( !empty( $options ) ) {
    if ( $product && taxonomy_exists( $attribute ) ) {
      $terms = wc_get_product_terms( VAM_UDesign_WC_Compat::get_product_id( $product ), $attribute, array( 'fields' => 'all', 'orderby' => 'menu_order' ) );
      foreach ( $terms as $term ) {
        if ( in_array( $term->slug, $options ) ) {
          $swatch_term = new VAM_UDesign_WC_Swatch_Term( $term->term_id, $attribute, $args['selected'] == $term->slug );
          $swatch_html .= $swatch_term->get_html();
        }
      }
    }
  }

  $swatch_html .= '</div>';

  echo apply_filters( 'vam_udesign_wc_swatches_variation_attribute_options_html', $swatch_html, $args );
}

/** Loop ******************************************************************/
if ( ! function_exists( 'vam_udesign_page_title' ) ) {

  /**
   * vam_udesign_page_title function.
   *
   * @param  bool $echo
   * @return string
   */
  function vam_udesign_page_title( $echo = true ) {

    if ( is_search() ) {
      $page_title = sprintf( __( 'Search Results: &ldquo;%s&rdquo;', 'vam-udesign' ), get_search_query() );

      if ( get_query_var( 'paged' ) )
        $page_title .= sprintf( __( '&nbsp;&ndash; Page %s', 'vam-udesign' ), get_query_var( 'paged' ) );

    } elseif ( is_tax() ) {

      $page_title = single_term_title( '', false );

    } else {

      $shop_page_id = vam_udesign_get_page_id( 'designs' );
      $page_title   = get_the_title( $shop_page_id );

    }

    $page_title = apply_filters( 'vam_udesign_page_title', $page_title );

    if ( $echo )
      echo $page_title;
    else
      return $page_title;
  }
}

if ( ! function_exists( 'vam_udesign_design_loop_start' ) ) {

  /**
   * Output the start of a design loop. By default this is a UL.
   *
   * @param bool $echo
   * @return string
   */
  function vam_udesign_design_loop_start( $echo = true ) {
    ob_start();
    $GLOBALS['udesign_loop']['loop'] = 0;
    vam_udesign_get_template( 'loop/loop-start.php' );
    if ( $echo )
      echo ob_get_clean();
    else
      return ob_get_clean();
  }
}

if ( ! function_exists( 'vam_udesign_design_loop_end' ) ) {

  /**
   * Output the end of a design loop. By default this is a UL.
   *
   * @param bool $echo
   * @return string
   */
  function vam_udesign_design_loop_end( $echo = true ) {
    ob_start();

    vam_udesign_get_template( 'loop/loop-end.php' );

    if ( $echo )
      echo ob_get_clean();
    else
      return ob_get_clean();
  }
}

if (  ! function_exists( 'vam_udesign_template_loop_design_title' ) ) {

  /**
   * Show the design title in the design loop. By default this is an H3.
   */
  function vam_udesign_template_loop_design_title() {
    echo '<h3>' . get_the_title() . '</h3>';
  }
}

if (  ! function_exists( 'vam_udesign_template_loop_category_title' ) ) {

  /**
   * Show the subcategory title in the design loop.
   */
  function vam_udesign_template_loop_category_title( $category ) {
    ?>
    <h3>
      <?php
        echo $category->name;

        if ( $category->count > 0 )
          echo apply_filters( 'vam_udesign_subcategory_count_html', ' <mark class="count">(' . $category->count . ')</mark>', $category );
      ?>
    </h3>
    <?php
  }
}

if ( ! function_exists( 'vam_udesign_template_loop_design_link_open' ) ) {

  /**
   * Insert the opening anchor tag for designs in the loop.
   */
  function vam_udesign_template_loop_design_link_open() {
    echo '<a href="' . get_the_permalink() . '" class="udesign-loop-design-link">';
  }
}

if ( ! function_exists( 'vam_udesign_template_loop_design_link_close' ) ) {

  /**
   * Insert the opening anchor tag for designs in the loop.
   */
  function vam_udesign_template_loop_design_link_close() {
    echo '</a>';
  }
}

if ( ! function_exists( 'vam_udesign_template_loop_category_link_open' ) ) {

  /**
   * Insert the opening anchor tag for categories in the loop.
   */
  function vam_udesign_template_loop_category_link_open( $category ) {
    echo '<a href="' . get_term_link( $category, 'design_cat' ) . '">';
  }
}

if ( ! function_exists( 'vam_udesign_template_loop_category_link_close' ) ) {

  /**
   * Insert the opening anchor tag for categories in the loop.
   */
  function vam_udesign_template_loop_category_link_close() {
    echo '</a>';
  }
}

if ( ! function_exists( 'vam_udesign_taxonomy_archive_description' ) ) {

  /**
   * Show an archive description on taxonomy archives.
   */
  function vam_udesign_taxonomy_archive_description() {
    if ( is_tax( array( 'design_cat', 'design_tag' ) ) && 0 === absint( get_query_var( 'paged' ) ) ) {
      $description = wc_format_content( term_description() );
      if ( $description ) {
        echo '<div class="term-description">' . $description . '</div>';
      }
    }
  }
}

if ( ! function_exists( 'vam_udesign_design_archive_description' ) ) {

  /**
   * Show a shop page description on design archives.
   */
  function vam_udesign_design_archive_description() {
    if ( is_post_type_archive( 'design' ) && 0 === absint( get_query_var( 'paged' ) ) ) {
      $designs_page = get_post( vam_udesign_get_page_id( 'designs' ) );
      if ( $designs_page ) {
        $description = wc_format_content( $designs_page->post_content );
        if ( $description ) {
          echo '<div class="page-description">' . $description . '</div>';
        }
      }
    }
  }
}

if ( ! function_exists( 'vam_udesign_template_loop_add_to_cart' ) ) {

  /**
   * Get the add to cart template for the design loop.
   */
  function vam_udesign_template_loop_add_to_cart( $args = array() ) {
    global $post;

    if ( $post ) {
      $defaults = array(
        'quantity' => 1,
        'class'    => implode( ' ', array_filter( array(
          'button',
          'product_type_design'
        ) ) )
      );

      $args = apply_filters( 'vam_udesign_loop_add_to_cart_args', wp_parse_args( $args, $defaults ), $post );

      vam_udesign_get_template( 'loop/add-to-cart.php', $args );
    }
  }
}

if ( ! function_exists( 'vam_udesign_template_loop_design_thumbnail' ) ) {

  /**
   * Get the design thumbnail for the loop.
   */
  function vam_udesign_template_loop_design_thumbnail() {
    echo vam_udesign_get_design_thumbnail();
  }
}

if ( ! function_exists( 'vam_udesign_get_design_thumbnail' ) ) {

  /**
   * Get the design thumbnail, or the placeholder if not set.
   *
   * @param string $size (default: 'udesign_design_catalog')
   * @return string
   */
  function vam_udesign_get_design_thumbnail( $size = 'udesign_design_catalog' ) {
    global $post, $design;
    $image_size = vam_udesign_get_image_size_array( $size );

    if ( has_post_thumbnail() ) {
      $props = vam_udesign_get_design_attachment_props( get_post_thumbnail_id(), $post );
      $src   = $design->get_archive_image();
      $color = $design->get_background_color();
      return get_the_post_thumbnail( $post->ID, $image_size, array(
        'src'    => $src,
        'srcset' => $src,
        'title'  => $props['title'],
        'alt'    => $props['alt'],
        'style'  => sprintf( 'background:%s;', $color )
      ) );
    } elseif ( wc_placeholder_img_src() ) {
      return wc_placeholder_img( $image_size );
    }
  }
}

if ( ! function_exists( 'vam_udesign_result_count' ) ) {

  /**
   * Output the result count text (Showing x - x of x results).
   */
  function vam_udesign_result_count() {
    vam_udesign_get_template( 'loop/result-count.php' );
  }
}

if ( ! function_exists( 'vam_udesign_catalog_ordering' ) ) {

  /**
   * Output the design sorting options.
   */
  function vam_udesign_catalog_ordering() {
    global $wp_query;

    if ( 1 === $wp_query->found_posts ) {
      return;
    }

    $orderby                 = isset( $_GET['orderby'] ) ? wc_clean( $_GET['orderby'] ) : apply_filters( 'vam_udesign_default_catalog_orderby', 'title' );
    $catalog_orderby_options = apply_filters( 'vam_udesign_catalog_orderby', array(
      'title'   => __( 'Sort by name: ascending', 'vam-udesign' ),
      'title-desc'  => __( 'Sort by name: descending', 'vam-udesign' ),
      'date'    => __( 'Sort by date: old to new', 'vam-udesign' ),
      'date-desc'   => __( 'Sort by date: new to old', 'vam-udesign' ),
    ) );

    vam_udesign_get_template( 'loop/orderby.php', array( 'catalog_orderby_options' => $catalog_orderby_options, 'orderby' => $orderby ) );
  }
}

/** Global ****************************************************************/

if ( ! function_exists( 'vam_udesign_output_content_wrapper' ) ) {

  /**
   * Output the start of the page wrapper.
   */
  function vam_udesign_output_content_wrapper() {
    vam_udesign_get_template( 'global/wrapper-start.php' );
  }
}

if ( ! function_exists( 'vam_udesign_output_content_wrapper_end' ) ) {

  /**
   * Output the end of the page wrapper.
   */
  function vam_udesign_output_content_wrapper_end() {
    vam_udesign_get_template( 'global/wrapper-end.php' );
  }
}

if ( ! function_exists( 'vam_udesign_get_sidebar' ) ) {

  /**
   * Get the shop sidebar template.
   */
  function vam_udesign_get_sidebar() {
    vam_udesign_get_template( 'global/sidebar.php' );
  }
}

/** Single Design ********************************************************/

if (  ! function_exists( 'vam_udesign_print_notices_wrapper' ) ) {

  /**
   * Show wrapper for messages.
   */
  function vam_udesign_print_notices_wrapper() {
    echo '<div class="vam_udesign_messages"></div>';
  }
}
add_action( 'vam_udesign_before_single_design', 'vam_udesign_print_notices_wrapper' );
add_action( 'vam_udesign_before_designer', 'vam_udesign_print_notices_wrapper' );

if ( ! function_exists( 'vam_udesign_show_design_images' ) ) {

  /**
   * Output the design image before the single design summary.
   */
  function vam_udesign_show_design_images() {
    vam_udesign_get_template( 'single-design/design-image.php' );
  }
}

if ( ! function_exists( 'vam_udesign_template_single_title' ) ) {

  /**
   * Output the design title.
   */
  function vam_udesign_template_single_title() {
    return woocommerce_template_single_title();
  }
}

if ( ! function_exists( 'vam_udesign_template_single_price' ) ) {

  /**
   * Output the design price.
   */
  function vam_udesign_template_single_price() {
    vam_udesign_get_template( 'single-design/price.php' );
  }
}

if ( ! function_exists( 'vam_udesign_template_single_excerpt' ) ) {

  /**
   * Output the design short description (excerpt).
   */
  function vam_udesign_template_single_excerpt() {
    vam_udesign_get_template( 'single-design/short-description.php' );
  }
}

if ( ! function_exists( 'vam_udesign_template_single_meta' ) ) {

  /**
   * Output the design meta.
   */
  function vam_udesign_template_single_meta() {
    vam_udesign_get_template( 'single-design/meta.php' );
  }
}

if ( ! function_exists( 'vam_udesign_template_single_sharing' ) ) {

  /**
   * Output the design sharing.
   */
  function vam_udesign_template_single_sharing()  {
    vam_udesign_get_template( 'single-design/share.php' );
  }
}


if ( ! function_exists( 'vam_udesign_template_single_products_filter' ) ) {

  /**
   * Output a filter for woocommerce products.
   */
  function vam_udesign_template_single_products_filter()  {
    vam_udesign_get_template( 'single-design/products-filter.php', array() );
  }
}

if ( ! function_exists( 'vam_udesign_template_single_products_gallery' ) ) {

  /**
   * Output a gallery of woocommerce products for this design.
   */
  function vam_udesign_template_single_products_gallery( $category_id = null ) {
    global $product;

    if ( empty( $category_id ) && 'carousel' === get_option( 'udesign_products_list_layout' ) ) {
      $categories = vam_udesign_get_designer_product_categories( array( 'hide_empty' => true, 'fields' => 'ids' ) );
      $category_id = reset( $categories );
    }

    $query = vam_udesign_get_products( array(
      'designer_products' => true,
      'category_id' => $category_id
    ) );
    $designer_product_ids = $query->posts;

    $product = wc_get_product( reset( $query->posts ) );

    vam_udesign_get_template( 'single-design/products-gallery.php', array(
      'designer_product_ids' => $designer_product_ids
    ) );
  }
}

if ( ! function_exists( 'vam_udesign_template_single_add_to_cart' ) ) {

  /**
   * Trigger the single design add to cart action.
   */
  function vam_udesign_template_single_add_to_cart() {
    global $product, $post;
    if ( empty( $product ) ) {
      return;
    }

    $design_id = 0;
    if ( !empty( $post ) && !empty( $post->ID ) && !empty( $post->post_type ) && $post->post_type == 'design' ) {
      $design_id = $post->ID;
    }

    $product_props = vam_udesign_get_design_product_props( $product, $design_id );

    // Load the template
    vam_udesign_get_template( 'single-design/add-to-cart/design.php', array(
      'available_variations' => $product_props->variations,
      'attributes'           => $product_props->attributes,
      'selected_attributes'  => $product_props->selected_attributes
    ) );
  }
}

if ( ! function_exists( 'vam_udesign_single_variation' ) ) {

  /**
   * Output placeholders for the single variation data.
   */
  function vam_udesign_single_variation() {
    echo '<div class="woocommerce-variation single_variation"></div>';
  }
}

if ( ! function_exists( 'vam_udesign_single_design_add_to_cart_button' ) ) {

  /**
   * Output the design add to cart button.
   */
  function vam_udesign_single_design_add_to_cart_button() {
    vam_udesign_get_template( 'single-design/add-to-cart/design-add-to-cart-button.php' );
  }
}

if ( ! function_exists( 'vam_udesign_single_design_buttons' ) ) {

  /**
   * Output the customize design button.
   *
   * @return void
   */
  function vam_udesign_single_design_buttons() {
    global $design;

    if ( !empty( $design ) && !empty( $design->get_disable_customize() ) ) {
      return;
    }

    if ( is_designer_page() ) {
      return;
    }

    if ( is_product() ) {
      global $product;
      if ( empty( $product ) ) {
        return;
      }

      $product_id = VAM_UDesign_WC_Compat::get_product_id( $product );
      if ( get_post_meta( $product_id, '_designer', true ) != 'yes' ) {
        return;
      }
    }

    vam_udesign_get_template( 'single-design/add-to-cart/design-buttons.php' );
  }
}

if ( ! function_exists( 'vam_udesign_output_design_data_tabs' ) ) {

  /**
   * Output the design tabs.
   */
  function vam_udesign_output_design_data_tabs() {
    vam_udesign_get_template( 'single-design/tabs/tabs.php' );
  }
}

if ( ! function_exists( 'vam_udesign_design_description_tab' ) ) {

  /**
   * Output the description tab content.
   */
  function vam_udesign_design_description_tab() {
    vam_udesign_get_template( 'single-design/tabs/description.php' );
  }
}

if ( ! function_exists( 'vam_udesign_default_design_tabs' ) ) {

  /**
   * Add default design tabs to design pages.
   *
   * @param array $tabs
   * @return array
   */
  function vam_udesign_default_design_tabs( $tabs = array() ) {
    global $design, $post;

    // Description tab - shows design content
    if ( $post->post_content ) {
      $tabs['description'] = array(
        'title'    => __( 'Description', 'vam-udesign' ),
        'priority' => 10,
        'callback' => 'vam_udesign_design_description_tab'
      );
    }

    return $tabs;
  }
}

if ( ! function_exists( 'vam_udesign_sort_design_tabs' ) ) {

  /**
   * Sort tabs by priority.
   *
   * @param array $tabs
   * @return array
   */
  function vam_udesign_sort_design_tabs( $tabs = array() ) {

    // Make sure the $tabs parameter is an array
    if ( ! is_array( $tabs ) ) {
      trigger_error( "Function vam_udesign_sort_design_tabs() expects an array as the first parameter. Defaulting to empty array." );
      $tabs = array( );
    }

    // Re-order tabs by priority
    if ( ! function_exists( '_sort_priority_callback' ) ) {
      function _sort_priority_callback( $a, $b ) {
        if ( $a['priority'] === $b['priority'] )
          return 0;
        return ( $a['priority'] < $b['priority'] ) ? -1 : 1;
      }
    }

    uasort( $tabs, '_sort_priority_callback' );

    return $tabs;
  }
}

if ( ! function_exists( 'vam_udesign_output_related_designs' ) ) {

  /**
   * Output the related designs.
   */
  function vam_udesign_output_related_designs() {

    $args = array(
      'posts_per_page'  => 4,
      'columns'         => 4,
      'orderby'         => 'rand'
    );

    vam_udesign_related_designs( apply_filters( 'vam_udesign_output_related_designs_args', $args ) );
  }
}

if ( ! function_exists( 'vam_udesign_related_designs' ) ) {

  /**
   * Output the related designs.
   *
   * @param array Provided arguments
   */
  function vam_udesign_related_designs( $args = array() ) {
    $defaults = array(
      'posts_per_page' => 4,
      'columns'        => 4,
      'orderby'        => 'rand'
    );

    $args = wp_parse_args( $args, $defaults );

    vam_udesign_get_template( 'single-design/related.php', $args );
  }
}

/** Forms ****************************************************************/

if ( ! function_exists( 'get_design_search_form' ) ) {

  /**
   * Display design search form.
   *
   * Will first attempt to locate the design-searchform.php file in either the child or.
   * the parent, then load it. If it doesn't exist, then the default search form.
   * will be displayed.
   *
   * The default searchform uses html5.
   *
   * @param bool $echo (default: true)
   * @return string
   */
  function get_design_search_form( $echo = true  ) {
    ob_start();

    do_action( 'pre_get_design_search_form'  );

    vam_udesign_get_template( 'design-searchform.php' );

    $form = apply_filters( 'get_design_search_form', ob_get_clean() );

    if ( $echo ) {
      echo $form;
    } else {
      return $form;
    }
  }
}

if ( ! function_exists( 'vam_udesign_product_dropdown_categories' ) ) {

  /**
   * Dropdown categories.
   *
   * @param array $args
   * @return string
   */
  function vam_udesign_product_dropdown_categories( $args = array() ) {
    global $wp_query;

    $terms = array();
    if ( !empty( $args['designer_products'] ) ) {
      $terms = vam_udesign_get_designer_product_categories( $args );
    } else {
      $terms = get_terms( 'product_cat', apply_filters( 'wc_product_dropdown_categories_get_terms_args', $args ) );
    }

    if ( empty( $terms ) ) {
      return;
    }

    $current_product_cat = isset( $wp_query->query_vars['product_cat'] ) ? $wp_query->query_vars['product_cat'] : '';
    $defaults            = array(
      'pad_counts'         => 1,
      'show_count'         => 1,
      'hierarchical'       => 1,
      'hide_empty'         => 1,
      'show_all'           => 1,
      'show_uncategorized' => 1,
      'orderby'            => 'name',
      'selected'           => $current_product_cat,
      'menu_order'         => false
    );

    $args = wp_parse_args( $args, $defaults );

    if ( $args['orderby'] == 'order' ) {
      $args['orderby']    = 'name';
      $args['menu_order'] = 'ASC';
    }

    $output  = "<select name='product_cat' class='dropdown_product_cat'>";
    if ( $args['show_all'] ) {
      $output .= '<option value="" ' .  selected( $current_product_cat, '', false ) . '>' . __( 'All categories', 'vam-udesign' ) . '</option>';
    }
    $output .= wc_walk_category_dropdown_tree( $terms, 0, $args );
    if ( $args['show_uncategorized'] ) {
      $output .= '<option value="0" ' . selected( $current_product_cat, '0', false ) . '>' . __( 'Uncategorized', 'vam-udesign' ) . '</option>';
    }
    $output .= "</select>";

    echo $output;
  }
}


if ( ! function_exists( 'vam_udesign_design_dropdown_categories' ) ) {

  /**
   * Dropdown categories.
   *
   * @param array $args
   * @return string
   */
  function vam_udesign_design_dropdown_categories( $args = array() ) {
    global $wp_query;
    $terms = get_terms( 'design_cat', apply_filters( 'vam_udesign_design_dropdown_categories_get_terms_args', $args ) );

    if ( empty( $terms ) ) {
      return;
    }

    $current_design_cat = isset( $wp_query->query_vars['design_cat'] ) ? $wp_query->query_vars['design_cat'] : '';
    $defaults            = array(
      'pad_counts'         => 1,
      'show_count'         => 1,
      'hierarchical'       => 1,
      'hide_empty'         => 1,
      'show_all'           => 1,
      'show_uncategorized' => 1,
      'orderby'            => 'name',
      'selected'           => $current_design_cat,
      'menu_order'         => false
    );

    $args = wp_parse_args( $args, $defaults );

    if ( $args['orderby'] == 'order' ) {
      $args['menu_order'] = 'asc';
      $args['orderby']    = 'name';
    }

    $output  = "<select name='design_cat' class='dropdown_design_cat'>";
    if ( $args['show_all'] ) {
      $output .= '<option value="" ' .  selected( $current_design_cat, '', false ) . '>' . __( 'All categories', 'vam-udesign' ) . '</option>';
    }
    $output .= wc_walk_category_dropdown_tree( $terms, 0, $args );
    if ( $args['show_uncategorized'] ) {
      $output .= '<option value="0" ' . selected( $current_design_cat, '0', false ) . '>' . __( 'Uncategorized', 'vam-udesign' ) . '</option>';
    }
    $output .= "</select>";

    echo $output;
  }
}

if ( ! function_exists( 'vam_udesign_get_designer_product' ) ) {

  /**
   * Get the designer variable product.
   */
  function vam_udesign_get_designer_product() {
    global $product;

    if ( !empty( $product ) ) {
      return $product;
    }

    if ( !empty( $_REQUEST['product_id'] ) ) {
      $product_id = absint( $_REQUEST['product_id'] );
    }

    if ( empty( $product_id ) ) {
      $product_id = absint( get_option( 'udesign_default_product_id' ) );
    }

    if ( empty( $product_id ) ) {
      return false;
    }

    if ( 'product_variation' == get_post_type( $product_id ) ) {
      $product_id = absint( wp_get_post_parent_id( $product_id ) );
    }

    if ( !empty( $product_id ) ) {
      $product = wc_get_product( $product_id );
    }

    if ( empty( $product ) ) {
      return false;
    }

    return $product;
  }
}

if ( ! function_exists( 'vam_udesign_photoswipe' ) ) {

  /**
   * Get the shop sidebar template.
   *
   */
  function vam_udesign_photoswipe() {
    vam_udesign_get_template( 'single-design/photoswipe.php' );
  }
}

if ( ! function_exists( 'vam_udesign_setup_loop' ) ) {

  /**
   * Setup loop.
   *
   */
  function vam_udesign_setup_loop() {
    if ( !empty( $GLOBALS['wp_query'] ) ) {
      $GLOBALS['wp_query']->set( 'wc_query', 'product_query' );
    }
  }
}
